"""DynamoDB Tool for PraisonAI Agents.

AWS DynamoDB operations.

Usage:
    from praisonai_tools import DynamoDBTool
    
    dynamo = DynamoDBTool()
    items = dynamo.scan(table="users")

Environment Variables:
    AWS_ACCESS_KEY_ID: AWS access key
    AWS_SECRET_ACCESS_KEY: AWS secret key
    AWS_REGION: AWS region
"""

import os
import logging
from typing import Any, Dict, List, Optional, Union

from praisonai_tools.tools.base import BaseTool

logger = logging.getLogger(__name__)


class DynamoDBTool(BaseTool):
    """Tool for DynamoDB operations."""
    
    name = "dynamodb"
    description = "AWS DynamoDB database operations."
    
    def __init__(self, region: Optional[str] = None):
        self.region = region or os.getenv("AWS_REGION", "us-east-1")
        self._client = None
        super().__init__()
    
    @property
    def client(self):
        if self._client is None:
            try:
                import boto3
            except ImportError:
                raise ImportError("boto3 not installed. Install with: pip install boto3")
            self._client = boto3.client("dynamodb", region_name=self.region)
        return self._client
    
    def run(
        self,
        action: str = "scan",
        table: Optional[str] = None,
        **kwargs
    ) -> Union[str, Dict[str, Any], List[Dict[str, Any]]]:
        action = action.lower().replace("-", "_")
        
        if action == "scan":
            return self.scan(table=table, **kwargs)
        elif action == "get_item":
            return self.get_item(table=table, key=kwargs.get("key"))
        elif action == "put_item":
            return self.put_item(table=table, item=kwargs.get("item"))
        elif action == "delete_item":
            return self.delete_item(table=table, key=kwargs.get("key"))
        elif action == "query":
            return self.query(table=table, **kwargs)
        elif action == "list_tables":
            return self.list_tables()
        else:
            return {"error": f"Unknown action: {action}"}
    
    def scan(self, table: str, limit: int = 100) -> List[Dict[str, Any]]:
        """Scan table."""
        if not table:
            return [{"error": "table is required"}]
        
        try:
            resp = self.client.scan(TableName=table, Limit=limit)
            return resp.get("Items", [])
        except Exception as e:
            logger.error(f"DynamoDB scan error: {e}")
            return [{"error": str(e)}]
    
    def get_item(self, table: str, key: Dict) -> Dict[str, Any]:
        """Get item by key."""
        if not table or not key:
            return {"error": "table and key are required"}
        
        try:
            resp = self.client.get_item(TableName=table, Key=key)
            return resp.get("Item", {})
        except Exception as e:
            logger.error(f"DynamoDB get_item error: {e}")
            return {"error": str(e)}
    
    def put_item(self, table: str, item: Dict) -> Dict[str, Any]:
        """Put item."""
        if not table or not item:
            return {"error": "table and item are required"}
        
        try:
            self.client.put_item(TableName=table, Item=item)
            return {"success": True}
        except Exception as e:
            logger.error(f"DynamoDB put_item error: {e}")
            return {"error": str(e)}
    
    def delete_item(self, table: str, key: Dict) -> Dict[str, Any]:
        """Delete item."""
        if not table or not key:
            return {"error": "table and key are required"}
        
        try:
            self.client.delete_item(TableName=table, Key=key)
            return {"success": True}
        except Exception as e:
            logger.error(f"DynamoDB delete_item error: {e}")
            return {"error": str(e)}
    
    def query(self, table: str, key_condition: str, expression_values: Dict) -> List[Dict[str, Any]]:
        """Query table."""
        if not table or not key_condition:
            return [{"error": "table and key_condition are required"}]
        
        try:
            resp = self.client.query(
                TableName=table,
                KeyConditionExpression=key_condition,
                ExpressionAttributeValues=expression_values,
            )
            return resp.get("Items", [])
        except Exception as e:
            logger.error(f"DynamoDB query error: {e}")
            return [{"error": str(e)}]
    
    def list_tables(self) -> List[Dict[str, Any]]:
        """List tables."""
        try:
            resp = self.client.list_tables()
            return [{"table_name": t} for t in resp.get("TableNames", [])]
        except Exception as e:
            logger.error(f"DynamoDB list_tables error: {e}")
            return [{"error": str(e)}]


def dynamodb_scan(table: str, limit: int = 100) -> List[Dict[str, Any]]:
    """Scan DynamoDB table."""
    return DynamoDBTool().scan(table=table, limit=limit)



"""SQLite Tool for PraisonAI Agents.

Execute queries on SQLite databases.

Usage:
    from praisonai_tools import SQLiteTool
    
    sqlite = SQLiteTool(database="mydb.sqlite")
    results = sqlite.query("SELECT * FROM users LIMIT 10")

Environment Variables:
    SQLITE_DATABASE: Path to SQLite database file
"""

import os
import logging
from typing import Any, Dict, List, Optional, Union

from praisonai_tools.tools.base import BaseTool

logger = logging.getLogger(__name__)


class SQLiteTool(BaseTool):
    """Tool for interacting with SQLite databases."""
    
    name = "sqlite"
    description = "Execute SQL queries on SQLite databases."
    
    def __init__(self, database: Optional[str] = None):
        self.database = database or os.getenv("SQLITE_DATABASE", ":memory:")
        self._conn = None
        super().__init__()
    
    def _get_connection(self):
        """Get database connection."""
        import sqlite3
        
        if self._conn is None:
            self._conn = sqlite3.connect(self.database)
            self._conn.row_factory = sqlite3.Row
        return self._conn
    
    def run(
        self,
        action: str = "query",
        sql: Optional[str] = None,
        table: Optional[str] = None,
        limit: int = 100,
        **kwargs
    ) -> Union[str, Dict[str, Any], List[Dict[str, Any]]]:
        action = action.lower().replace("-", "_")
        
        if action == "query":
            return self.query(sql=sql, limit=limit)
        elif action == "list_tables":
            return self.list_tables()
        elif action == "describe_table":
            return self.describe_table(table=table)
        elif action == "execute":
            return self.execute(sql=sql)
        else:
            return {"error": f"Unknown action: {action}"}
    
    def query(self, sql: str, limit: int = 100) -> List[Dict[str, Any]]:
        """Execute SELECT query."""
        if not sql:
            return [{"error": "SQL query is required"}]
        
        sql_upper = sql.strip().upper()
        if not sql_upper.startswith("SELECT"):
            return [{"error": "Only SELECT queries allowed."}]
        
        try:
            conn = self._get_connection()
            cursor = conn.cursor()
            
            if "LIMIT" not in sql_upper:
                sql = f"{sql.rstrip(';')} LIMIT {limit}"
            
            cursor.execute(sql)
            rows = cursor.fetchall()
            results = [dict(row) for row in rows]
            cursor.close()
            return results
        except Exception as e:
            logger.error(f"SQLite query error: {e}")
            return [{"error": str(e)}]
    
    def execute(self, sql: str) -> Dict[str, Any]:
        """Execute SQL statement."""
        if not sql:
            return {"error": "SQL statement is required"}
        
        try:
            conn = self._get_connection()
            cursor = conn.cursor()
            cursor.execute(sql)
            rowcount = cursor.rowcount
            conn.commit()
            cursor.close()
            return {"success": True, "rows_affected": rowcount}
        except Exception as e:
            logger.error(f"SQLite execute error: {e}")
            return {"error": str(e)}
    
    def list_tables(self) -> List[Dict[str, Any]]:
        """List all tables."""
        try:
            conn = self._get_connection()
            cursor = conn.cursor()
            cursor.execute("SELECT name FROM sqlite_master WHERE type='table' ORDER BY name")
            rows = cursor.fetchall()
            cursor.close()
            return [{"table_name": row["name"]} for row in rows]
        except Exception as e:
            logger.error(f"SQLite list_tables error: {e}")
            return [{"error": str(e)}]
    
    def describe_table(self, table: str) -> Dict[str, Any]:
        """Get table schema."""
        if not table:
            return {"error": "table name is required"}
        
        try:
            conn = self._get_connection()
            cursor = conn.cursor()
            cursor.execute(f"PRAGMA table_info({table})")
            rows = cursor.fetchall()
            cursor.close()
            
            if not rows:
                return {"error": f"Table '{table}' not found"}
            
            columns = []
            for row in rows:
                columns.append({
                    "name": row["name"],
                    "type": row["type"],
                    "nullable": not row["notnull"],
                    "primary_key": bool(row["pk"]),
                    "default": row["dflt_value"],
                })
            
            return {"table": table, "columns": columns}
        except Exception as e:
            logger.error(f"SQLite describe_table error: {e}")
            return {"error": str(e)}
    
    def close(self):
        if self._conn:
            self._conn.close()
            self._conn = None


def query_sqlite(sql: str, database: str = ":memory:") -> List[Dict[str, Any]]:
    """Execute SQLite query."""
    return SQLiteTool(database=database).query(sql=sql)



"""Shell Tool for PraisonAI Agents.

Execute shell commands.

Usage:
    from praisonai_tools import ShellTool
    
    shell = ShellTool()
    result = shell.execute("ls -la")
"""

import subprocess
import logging
from typing import Any, Dict, List, Optional, Union

from praisonai_tools.tools.base import BaseTool

logger = logging.getLogger(__name__)


class ShellTool(BaseTool):
    """Tool for executing shell commands."""
    
    name = "shell"
    description = "Execute shell commands and return output."
    
    def __init__(
        self,
        working_dir: Optional[str] = None,
        timeout: int = 60,
    ):
        self.working_dir = working_dir
        self.timeout = timeout
        super().__init__()
    
    def run(
        self,
        action: str = "execute",
        command: Optional[str] = None,
        args: Optional[List[str]] = None,
        **kwargs
    ) -> Union[str, Dict[str, Any]]:
        action = action.lower().replace("-", "_")
        
        if action == "execute":
            return self.execute(command=command, args=args)
        else:
            return {"error": f"Unknown action: {action}"}
    
    def execute(
        self,
        command: Optional[str] = None,
        args: Optional[List[str]] = None,
        tail: int = 100,
    ) -> Dict[str, Any]:
        """Execute a shell command."""
        if not command and not args:
            return {"error": "command or args is required"}
        
        try:
            if args:
                cmd = args
            else:
                cmd = command
                
            result = subprocess.run(
                cmd,
                shell=isinstance(cmd, str),
                capture_output=True,
                text=True,
                cwd=self.working_dir,
                timeout=self.timeout,
            )
            
            stdout = result.stdout
            if tail and stdout:
                lines = stdout.split("\n")
                stdout = "\n".join(lines[-tail:])
            
            return {
                "success": result.returncode == 0,
                "return_code": result.returncode,
                "stdout": stdout,
                "stderr": result.stderr if result.returncode != 0 else "",
            }
        except subprocess.TimeoutExpired:
            return {"error": f"Command timed out after {self.timeout}s"}
        except Exception as e:
            logger.error(f"Shell execute error: {e}")
            return {"error": str(e)}


def shell_execute(command: str) -> Dict[str, Any]:
    """Execute shell command."""
    return ShellTool().execute(command=command)



"""SearxNG Tool for PraisonAI Agents.

Privacy-respecting metasearch using SearxNG.

Usage:
    from praisonai_tools import SearxNGTool
    
    searx = SearxNGTool()
    results = searx.search("AI news")

Environment Variables:
    SEARXNG_URL: SearxNG instance URL
"""

import os
import logging
from typing import Any, Dict, List, Optional, Union

from praisonai_tools.tools.base import BaseTool

logger = logging.getLogger(__name__)


class SearxNGTool(BaseTool):
    """Tool for SearxNG metasearch."""
    
    name = "searxng"
    description = "Privacy-respecting metasearch using SearxNG."
    
    def __init__(self, url: Optional[str] = None):
        self.url = url or os.getenv("SEARXNG_URL", "https://searx.be")
        super().__init__()
    
    def run(
        self,
        action: str = "search",
        query: Optional[str] = None,
        categories: Optional[str] = None,
        **kwargs
    ) -> Union[str, Dict[str, Any], List[Dict[str, Any]]]:
        if action == "search":
            return self.search(query=query, categories=categories, **kwargs)
        return {"error": f"Unknown action: {action}"}
    
    def search(self, query: str, categories: str = "general", num: int = 10) -> List[Dict[str, Any]]:
        """Search SearxNG."""
        if not query:
            return [{"error": "query is required"}]
        
        try:
            import requests
        except ImportError:
            return [{"error": "requests not installed"}]
        
        try:
            params = {"q": query, "format": "json", "categories": categories}
            resp = requests.get(f"{self.url}/search", params=params, timeout=10)
            data = resp.json()
            
            results = []
            for item in data.get("results", [])[:num]:
                results.append({
                    "title": item.get("title"),
                    "url": item.get("url"),
                    "content": item.get("content"),
                    "engine": item.get("engine"),
                })
            return results
        except Exception as e:
            logger.error(f"SearxNG search error: {e}")
            return [{"error": str(e)}]


def searxng_search(query: str, num: int = 10) -> List[Dict[str, Any]]:
    """Search with SearxNG."""
    return SearxNGTool().search(query=query, num=num)



"""Chroma Vector DB Tool for PraisonAI Agents.

Vector database operations using ChromaDB.

Usage:
    from praisonai_tools import ChromaTool
    
    chroma = ChromaTool()
    chroma.add(collection="docs", documents=["text1", "text2"], ids=["1", "2"])

Environment Variables:
    CHROMA_HOST: Chroma server host (for client mode)
    CHROMA_PORT: Chroma server port
"""

import os
import logging
from typing import Any, Dict, List, Optional, Union

from praisonai_tools.tools.base import BaseTool

logger = logging.getLogger(__name__)


class ChromaTool(BaseTool):
    """Tool for ChromaDB vector database."""
    
    name = "chroma"
    description = "Store and query vectors in ChromaDB."
    
    def __init__(
        self,
        host: Optional[str] = None,
        port: Optional[int] = None,
        persist_directory: Optional[str] = None,
    ):
        self.host = host or os.getenv("CHROMA_HOST")
        self.port = port or int(os.getenv("CHROMA_PORT", "8000")) if os.getenv("CHROMA_PORT") else None
        self.persist_directory = persist_directory
        self._client = None
        super().__init__()
    
    @property
    def client(self):
        if self._client is None:
            try:
                import chromadb
            except ImportError:
                raise ImportError("chromadb not installed. Install with: pip install chromadb")
            
            if self.host and self.port:
                self._client = chromadb.HttpClient(host=self.host, port=self.port)
            elif self.persist_directory:
                self._client = chromadb.PersistentClient(path=self.persist_directory)
            else:
                self._client = chromadb.Client()
        return self._client
    
    def run(
        self,
        action: str = "query",
        collection: Optional[str] = None,
        query_texts: Optional[List[str]] = None,
        query_embeddings: Optional[List[List[float]]] = None,
        documents: Optional[List[str]] = None,
        embeddings: Optional[List[List[float]]] = None,
        ids: Optional[List[str]] = None,
        n_results: int = 10,
        **kwargs
    ) -> Union[str, Dict[str, Any], List[Dict[str, Any]]]:
        action = action.lower().replace("-", "_")
        
        if action == "query":
            return self.query(collection=collection, query_texts=query_texts, query_embeddings=query_embeddings, n_results=n_results)
        elif action == "add":
            return self.add(collection=collection, documents=documents, embeddings=embeddings, ids=ids, **kwargs)
        elif action == "delete":
            return self.delete(collection=collection, ids=ids)
        elif action == "get":
            return self.get(collection=collection, ids=ids)
        elif action == "list_collections":
            return self.list_collections()
        elif action == "create_collection":
            return self.create_collection(collection=collection)
        else:
            return {"error": f"Unknown action: {action}"}
    
    def query(
        self,
        collection: str,
        query_texts: Optional[List[str]] = None,
        query_embeddings: Optional[List[List[float]]] = None,
        n_results: int = 10,
    ) -> List[Dict[str, Any]]:
        """Query similar documents."""
        if not collection:
            return [{"error": "collection is required"}]
        if not query_texts and not query_embeddings:
            return [{"error": "query_texts or query_embeddings required"}]
        
        try:
            coll = self.client.get_collection(collection)
            
            params = {"n_results": n_results}
            if query_texts:
                params["query_texts"] = query_texts
            if query_embeddings:
                params["query_embeddings"] = query_embeddings
            
            results = coll.query(**params)
            
            items = []
            for i, doc_id in enumerate(results.get("ids", [[]])[0]):
                items.append({
                    "id": doc_id,
                    "document": results.get("documents", [[]])[0][i] if results.get("documents") else None,
                    "distance": results.get("distances", [[]])[0][i] if results.get("distances") else None,
                    "metadata": results.get("metadatas", [[]])[0][i] if results.get("metadatas") else None,
                })
            return items
        except Exception as e:
            logger.error(f"Chroma query error: {e}")
            return [{"error": str(e)}]
    
    def add(
        self,
        collection: str,
        documents: Optional[List[str]] = None,
        embeddings: Optional[List[List[float]]] = None,
        ids: List[str] = None,
        metadatas: Optional[List[Dict]] = None,
    ) -> Dict[str, Any]:
        """Add documents to collection."""
        if not collection or not ids:
            return {"error": "collection and ids are required"}
        if not documents and not embeddings:
            return {"error": "documents or embeddings required"}
        
        try:
            coll = self.client.get_or_create_collection(collection)
            
            params = {"ids": ids}
            if documents:
                params["documents"] = documents
            if embeddings:
                params["embeddings"] = embeddings
            if metadatas:
                params["metadatas"] = metadatas
            
            coll.add(**params)
            return {"success": True, "added_count": len(ids)}
        except Exception as e:
            logger.error(f"Chroma add error: {e}")
            return {"error": str(e)}
    
    def delete(self, collection: str, ids: List[str]) -> Dict[str, Any]:
        """Delete documents by ID."""
        if not collection or not ids:
            return {"error": "collection and ids are required"}
        
        try:
            coll = self.client.get_collection(collection)
            coll.delete(ids=ids)
            return {"success": True, "deleted": len(ids)}
        except Exception as e:
            logger.error(f"Chroma delete error: {e}")
            return {"error": str(e)}
    
    def get(self, collection: str, ids: Optional[List[str]] = None) -> List[Dict[str, Any]]:
        """Get documents by ID."""
        if not collection:
            return [{"error": "collection is required"}]
        
        try:
            coll = self.client.get_collection(collection)
            results = coll.get(ids=ids) if ids else coll.get()
            
            items = []
            for i, doc_id in enumerate(results.get("ids", [])):
                items.append({
                    "id": doc_id,
                    "document": results.get("documents", [])[i] if results.get("documents") else None,
                    "metadata": results.get("metadatas", [])[i] if results.get("metadatas") else None,
                })
            return items
        except Exception as e:
            logger.error(f"Chroma get error: {e}")
            return [{"error": str(e)}]
    
    def list_collections(self) -> List[Dict[str, Any]]:
        """List all collections."""
        try:
            collections = self.client.list_collections()
            return [{"name": c.name} for c in collections]
        except Exception as e:
            logger.error(f"Chroma list_collections error: {e}")
            return [{"error": str(e)}]
    
    def create_collection(self, collection: str) -> Dict[str, Any]:
        """Create a collection."""
        if not collection:
            return {"error": "collection is required"}
        
        try:
            self.client.create_collection(collection)
            return {"success": True, "collection": collection}
        except Exception as e:
            logger.error(f"Chroma create_collection error: {e}")
            return {"error": str(e)}


def chroma_query(collection: str, query_texts: List[str], n_results: int = 10) -> List[Dict[str, Any]]:
    """Query ChromaDB."""
    return ChromaTool().query(collection=collection, query_texts=query_texts, n_results=n_results)



"""Weaviate Vector DB Tool for PraisonAI Agents.

Vector database operations using Weaviate.

Usage:
    from praisonai_tools import WeaviateTool
    
    weaviate = WeaviateTool()
    results = weaviate.search(class_name="Document", query="machine learning")

Environment Variables:
    WEAVIATE_URL: Weaviate server URL
    WEAVIATE_API_KEY: Weaviate API key (optional)
"""

import os
import logging
from typing import Any, Dict, List, Optional, Union

from praisonai_tools.tools.base import BaseTool

logger = logging.getLogger(__name__)


class WeaviateTool(BaseTool):
    """Tool for Weaviate vector database."""
    
    name = "weaviate"
    description = "Store and query vectors in Weaviate."
    
    def __init__(
        self,
        url: Optional[str] = None,
        api_key: Optional[str] = None,
    ):
        self.url = url or os.getenv("WEAVIATE_URL", "http://localhost:8080")
        self.api_key = api_key or os.getenv("WEAVIATE_API_KEY")
        self._client = None
        super().__init__()
    
    @property
    def client(self):
        if self._client is None:
            try:
                import weaviate
            except ImportError:
                raise ImportError("weaviate-client not installed. Install with: pip install weaviate-client")
            
            if self.api_key:
                self._client = weaviate.Client(
                    url=self.url,
                    auth_client_secret=weaviate.AuthApiKey(api_key=self.api_key),
                )
            else:
                self._client = weaviate.Client(url=self.url)
        return self._client
    
    def run(
        self,
        action: str = "search",
        class_name: Optional[str] = None,
        query: Optional[str] = None,
        vector: Optional[List[float]] = None,
        data: Optional[Dict] = None,
        limit: int = 10,
        **kwargs
    ) -> Union[str, Dict[str, Any], List[Dict[str, Any]]]:
        action = action.lower().replace("-", "_")
        
        if action == "search":
            return self.search(class_name=class_name, query=query, limit=limit)
        elif action == "vector_search":
            return self.vector_search(class_name=class_name, vector=vector, limit=limit)
        elif action == "add":
            return self.add(class_name=class_name, data=data)
        elif action == "get_schema":
            return self.get_schema()
        else:
            return {"error": f"Unknown action: {action}"}
    
    def search(self, class_name: str, query: str, limit: int = 10) -> List[Dict[str, Any]]:
        """Search with text query."""
        if not class_name or not query:
            return [{"error": "class_name and query are required"}]
        
        try:
            result = (
                self.client.query
                .get(class_name, ["*"])
                .with_near_text({"concepts": [query]})
                .with_limit(limit)
                .do()
            )
            
            data = result.get("data", {}).get("Get", {}).get(class_name, [])
            return data
        except Exception as e:
            logger.error(f"Weaviate search error: {e}")
            return [{"error": str(e)}]
    
    def vector_search(self, class_name: str, vector: List[float], limit: int = 10) -> List[Dict[str, Any]]:
        """Search with vector."""
        if not class_name or not vector:
            return [{"error": "class_name and vector are required"}]
        
        try:
            result = (
                self.client.query
                .get(class_name, ["*"])
                .with_near_vector({"vector": vector})
                .with_limit(limit)
                .do()
            )
            
            data = result.get("data", {}).get("Get", {}).get(class_name, [])
            return data
        except Exception as e:
            logger.error(f"Weaviate vector_search error: {e}")
            return [{"error": str(e)}]
    
    def add(self, class_name: str, data: Dict) -> Dict[str, Any]:
        """Add object."""
        if not class_name or not data:
            return {"error": "class_name and data are required"}
        
        try:
            uuid = self.client.data_object.create(data, class_name)
            return {"success": True, "uuid": uuid}
        except Exception as e:
            logger.error(f"Weaviate add error: {e}")
            return {"error": str(e)}
    
    def get_schema(self) -> Dict[str, Any]:
        """Get schema."""
        try:
            return self.client.schema.get()
        except Exception as e:
            logger.error(f"Weaviate get_schema error: {e}")
            return {"error": str(e)}


def weaviate_search(class_name: str, query: str, limit: int = 10) -> List[Dict[str, Any]]:
    """Search Weaviate."""
    return WeaviateTool().search(class_name=class_name, query=query, limit=limit)



"""ArXiv Tool for PraisonAI Agents.

Search and retrieve academic papers from ArXiv.

Usage:
    from praisonai_tools import ArxivTool
    
    arxiv = ArxivTool()
    papers = arxiv.search("machine learning")
"""

import logging
from typing import Any, Dict, List, Optional, Union

from praisonai_tools.tools.base import BaseTool

logger = logging.getLogger(__name__)


class ArxivTool(BaseTool):
    """Tool for searching ArXiv papers."""
    
    name = "arxiv"
    description = "Search and retrieve academic papers from ArXiv."
    
    def __init__(self):
        super().__init__()
    
    def run(
        self,
        action: str = "search",
        query: Optional[str] = None,
        paper_id: Optional[str] = None,
        max_results: int = 5,
        **kwargs
    ) -> Union[str, Dict[str, Any], List[Dict[str, Any]]]:
        action = action.lower().replace("-", "_")
        
        if action == "search":
            return self.search(query=query, max_results=max_results)
        elif action == "get_paper":
            return self.get_paper(paper_id=paper_id)
        else:
            return {"error": f"Unknown action: {action}"}
    
    def search(self, query: str, max_results: int = 5) -> List[Dict[str, Any]]:
        """Search ArXiv papers."""
        if not query:
            return [{"error": "query is required"}]
        
        try:
            import arxiv
        except ImportError:
            return [{"error": "arxiv not installed. Install with: pip install arxiv"}]
        
        try:
            search = arxiv.Search(
                query=query,
                max_results=max_results,
                sort_by=arxiv.SortCriterion.Relevance,
            )
            
            results = []
            for paper in search.results():
                results.append({
                    "id": paper.entry_id.split("/")[-1],
                    "title": paper.title,
                    "authors": [a.name for a in paper.authors[:5]],
                    "summary": paper.summary[:500],
                    "published": str(paper.published.date()),
                    "pdf_url": paper.pdf_url,
                    "categories": paper.categories,
                })
            return results
        except Exception as e:
            logger.error(f"ArXiv search error: {e}")
            return [{"error": str(e)}]
    
    def get_paper(self, paper_id: str) -> Dict[str, Any]:
        """Get paper by ID."""
        if not paper_id:
            return {"error": "paper_id is required"}
        
        try:
            import arxiv
        except ImportError:
            return {"error": "arxiv not installed"}
        
        try:
            search = arxiv.Search(id_list=[paper_id])
            paper = next(search.results(), None)
            
            if not paper:
                return {"error": f"Paper '{paper_id}' not found"}
            
            return {
                "id": paper.entry_id.split("/")[-1],
                "title": paper.title,
                "authors": [a.name for a in paper.authors],
                "summary": paper.summary,
                "published": str(paper.published.date()),
                "updated": str(paper.updated.date()),
                "pdf_url": paper.pdf_url,
                "categories": paper.categories,
                "primary_category": paper.primary_category,
            }
        except Exception as e:
            logger.error(f"ArXiv get_paper error: {e}")
            return {"error": str(e)}


def arxiv_search(query: str, max_results: int = 5) -> List[Dict[str, Any]]:
    """Search ArXiv papers."""
    return ArxivTool().search(query=query, max_results=max_results)



"""Email Tool for PraisonAI Agents.

Provides email sending and reading capabilities via SMTP/IMAP.
Supports multiple providers (Gmail, Outlook, custom SMTP).

Usage:
    from praisonai_tools import EmailTool
    
    # Simple usage with Gmail
    email = EmailTool(
        smtp_host="smtp.gmail.com",
        smtp_port=587,
        username="your@gmail.com",
        password="your-app-password"  # Use App Password, not regular password
    )
    
    # Send email
    result = email.send(
        to="recipient@example.com",
        subject="Hello",
        body="This is a test email"
    )

Environment Variables:
    EMAIL_SMTP_HOST: SMTP server hostname
    EMAIL_SMTP_PORT: SMTP server port (default: 587)
    EMAIL_USERNAME: Email username/address
    EMAIL_PASSWORD: Email password or app password
    EMAIL_IMAP_HOST: IMAP server hostname (for reading)
    EMAIL_IMAP_PORT: IMAP server port (default: 993)
"""

import os
import logging
from typing import Any, Dict, List, Optional, Union
from email.mime.text import MIMEText
from email.mime.multipart import MIMEMultipart
from email.mime.base import MIMEBase
from email import encoders

from praisonai_tools.tools.base import BaseTool

logger = logging.getLogger(__name__)


class EmailTool(BaseTool):
    """Tool for sending and reading emails.
    
    Supports SMTP for sending and IMAP for reading emails.
    Works with Gmail, Outlook, and custom SMTP/IMAP servers.
    
    Attributes:
        name: Tool identifier
        description: Tool description for LLM
        smtp_host: SMTP server hostname
        smtp_port: SMTP server port
        imap_host: IMAP server hostname  
        imap_port: IMAP server port
        username: Email username
        password: Email password/app password
        use_tls: Whether to use TLS encryption
    """
    
    name = "email"
    description = "Send and read emails. Can send emails with attachments and read inbox messages."
    
    # Common provider configurations
    PROVIDERS = {
        "gmail": {
            "smtp_host": "smtp.gmail.com",
            "smtp_port": 587,
            "imap_host": "imap.gmail.com",
            "imap_port": 993,
        },
        "outlook": {
            "smtp_host": "smtp.office365.com",
            "smtp_port": 587,
            "imap_host": "outlook.office365.com",
            "imap_port": 993,
        },
        "yahoo": {
            "smtp_host": "smtp.mail.yahoo.com",
            "smtp_port": 587,
            "imap_host": "imap.mail.yahoo.com",
            "imap_port": 993,
        },
    }
    
    def __init__(
        self,
        smtp_host: Optional[str] = None,
        smtp_port: Optional[int] = None,
        imap_host: Optional[str] = None,
        imap_port: Optional[int] = None,
        username: Optional[str] = None,
        password: Optional[str] = None,
        provider: Optional[str] = None,
        use_tls: bool = True,
        default_from: Optional[str] = None,
    ):
        """Initialize EmailTool.
        
        Args:
            smtp_host: SMTP server hostname (or use EMAIL_SMTP_HOST env var)
            smtp_port: SMTP server port (or use EMAIL_SMTP_PORT env var)
            imap_host: IMAP server hostname (or use EMAIL_IMAP_HOST env var)
            imap_port: IMAP server port (or use EMAIL_IMAP_PORT env var)
            username: Email username (or use EMAIL_USERNAME env var)
            password: Email password (or use EMAIL_PASSWORD env var)
            provider: Shortcut for common providers: "gmail", "outlook", "yahoo"
            use_tls: Whether to use TLS (default: True)
            default_from: Default sender address (defaults to username)
        """
        # Apply provider defaults if specified
        if provider and provider.lower() in self.PROVIDERS:
            config = self.PROVIDERS[provider.lower()]
            smtp_host = smtp_host or config["smtp_host"]
            smtp_port = smtp_port or config["smtp_port"]
            imap_host = imap_host or config["imap_host"]
            imap_port = imap_port or config["imap_port"]
        
        # Get from environment if not provided
        self.smtp_host = smtp_host or os.getenv("EMAIL_SMTP_HOST", "smtp.gmail.com")
        self.smtp_port = smtp_port or int(os.getenv("EMAIL_SMTP_PORT", "587"))
        self.imap_host = imap_host or os.getenv("EMAIL_IMAP_HOST", "imap.gmail.com")
        self.imap_port = imap_port or int(os.getenv("EMAIL_IMAP_PORT", "993"))
        self.username = username or os.getenv("EMAIL_USERNAME")
        self.password = password or os.getenv("EMAIL_PASSWORD")
        self.use_tls = use_tls
        self.default_from = default_from or self.username
        
        super().__init__()
    
    def run(
        self,
        action: str = "send",
        to: Optional[str] = None,
        subject: Optional[str] = None,
        body: Optional[str] = None,
        html: Optional[str] = None,
        cc: Optional[str] = None,
        bcc: Optional[str] = None,
        attachments: Optional[List[str]] = None,
        folder: str = "INBOX",
        limit: int = 10,
        unread_only: bool = False,
        **kwargs
    ) -> Union[str, Dict[str, Any]]:
        """Execute email action.
        
        Args:
            action: "send" to send email, "read" to read inbox, "search" to search
            to: Recipient email address(es), comma-separated
            subject: Email subject
            body: Plain text body
            html: HTML body (optional, takes precedence over body)
            cc: CC recipients, comma-separated
            bcc: BCC recipients, comma-separated
            attachments: List of file paths to attach
            folder: IMAP folder to read from (default: INBOX)
            limit: Maximum emails to return when reading
            unread_only: Only return unread emails
            
        Returns:
            Success message or email data
        """
        action = action.lower()
        
        if action == "send":
            return self.send(
                to=to,
                subject=subject,
                body=body,
                html=html,
                cc=cc,
                bcc=bcc,
                attachments=attachments
            )
        elif action == "read":
            return self.read(
                folder=folder,
                limit=limit,
                unread_only=unread_only
            )
        elif action == "search":
            query = kwargs.get("query", "")
            return self.search(query=query, folder=folder, limit=limit)
        else:
            return f"Unknown action: {action}. Use 'send', 'read', or 'search'."
    
    def send(
        self,
        to: str,
        subject: str,
        body: str,
        html: Optional[str] = None,
        cc: Optional[str] = None,
        bcc: Optional[str] = None,
        attachments: Optional[List[str]] = None,
        from_addr: Optional[str] = None,
    ) -> str:
        """Send an email.
        
        Args:
            to: Recipient email address(es), comma-separated
            subject: Email subject
            body: Plain text body
            html: HTML body (optional)
            cc: CC recipients, comma-separated
            bcc: BCC recipients, comma-separated
            attachments: List of file paths to attach
            from_addr: Sender address (defaults to username)
            
        Returns:
            Success or error message
        """
        if not self.username or not self.password:
            return "Error: Email credentials not configured. Set EMAIL_USERNAME and EMAIL_PASSWORD."
        
        if not to:
            return "Error: Recipient (to) is required."
        
        if not subject:
            return "Error: Subject is required."
        
        if not body and not html:
            return "Error: Body or HTML content is required."
        
        try:
            import smtplib
        except ImportError:
            return "Error: smtplib not available."
        
        try:
            # Create message
            if html or attachments:
                msg = MIMEMultipart("alternative" if html else "mixed")
                msg.attach(MIMEText(body or "", "plain"))
                if html:
                    msg.attach(MIMEText(html, "html"))
            else:
                msg = MIMEText(body, "plain")
            
            msg["Subject"] = subject
            msg["From"] = from_addr or self.default_from
            msg["To"] = to
            
            if cc:
                msg["Cc"] = cc
            if bcc:
                msg["Bcc"] = bcc
            
            # Add attachments
            if attachments:
                for filepath in attachments:
                    if os.path.exists(filepath):
                        with open(filepath, "rb") as f:
                            part = MIMEBase("application", "octet-stream")
                            part.set_payload(f.read())
                            encoders.encode_base64(part)
                            part.add_header(
                                "Content-Disposition",
                                f"attachment; filename={os.path.basename(filepath)}"
                            )
                            msg.attach(part)
                    else:
                        logger.warning(f"Attachment not found: {filepath}")
            
            # Build recipient list
            recipients = [addr.strip() for addr in to.split(",")]
            if cc:
                recipients.extend([addr.strip() for addr in cc.split(",")])
            if bcc:
                recipients.extend([addr.strip() for addr in bcc.split(",")])
            
            # Send email
            with smtplib.SMTP(self.smtp_host, self.smtp_port) as server:
                if self.use_tls:
                    server.starttls()
                server.login(self.username, self.password)
                server.sendmail(self.default_from, recipients, msg.as_string())
            
            logger.info(f"Email sent to {to}")
            return f"Email sent successfully to {to}"
            
        except smtplib.SMTPAuthenticationError:
            return "Error: Authentication failed. Check username and password."
        except smtplib.SMTPException as e:
            logger.error(f"SMTP error: {e}")
            return f"Error sending email: {str(e)}"
        except Exception as e:
            logger.error(f"Email error: {e}")
            return f"Error: {str(e)}"
    
    def read(
        self,
        folder: str = "INBOX",
        limit: int = 10,
        unread_only: bool = False,
    ) -> List[Dict[str, Any]]:
        """Read emails from inbox.
        
        Args:
            folder: IMAP folder to read from
            limit: Maximum number of emails to return
            unread_only: Only return unread emails
            
        Returns:
            List of email dictionaries with subject, from, date, body
        """
        if not self.username or not self.password:
            return [{"error": "Email credentials not configured."}]
        
        try:
            import imaplib
            import email
            from email.header import decode_header
        except ImportError:
            return [{"error": "imaplib not available."}]
        
        try:
            # Connect to IMAP server
            mail = imaplib.IMAP4_SSL(self.imap_host, self.imap_port)
            mail.login(self.username, self.password)
            mail.select(folder)
            
            # Search for emails
            search_criteria = "UNSEEN" if unread_only else "ALL"
            _, message_numbers = mail.search(None, search_criteria)
            
            emails = []
            message_ids = message_numbers[0].split()
            
            # Get latest emails (reverse order)
            for num in reversed(message_ids[-limit:]):
                _, msg_data = mail.fetch(num, "(RFC822)")
                
                for response_part in msg_data:
                    if isinstance(response_part, tuple):
                        msg = email.message_from_bytes(response_part[1])
                        
                        # Decode subject
                        subject, encoding = decode_header(msg["Subject"])[0]
                        if isinstance(subject, bytes):
                            subject = subject.decode(encoding or "utf-8")
                        
                        # Decode from
                        from_addr, encoding = decode_header(msg.get("From", ""))[0]
                        if isinstance(from_addr, bytes):
                            from_addr = from_addr.decode(encoding or "utf-8")
                        
                        # Get body
                        body = ""
                        if msg.is_multipart():
                            for part in msg.walk():
                                if part.get_content_type() == "text/plain":
                                    body = part.get_payload(decode=True).decode()
                                    break
                        else:
                            body = msg.get_payload(decode=True).decode()
                        
                        emails.append({
                            "subject": subject,
                            "from": from_addr,
                            "date": msg.get("Date", ""),
                            "body": body[:500] + "..." if len(body) > 500 else body
                        })
            
            mail.close()
            mail.logout()
            
            return emails
            
        except imaplib.IMAP4.error as e:
            logger.error(f"IMAP error: {e}")
            return [{"error": f"IMAP error: {str(e)}"}]
        except Exception as e:
            logger.error(f"Email read error: {e}")
            return [{"error": str(e)}]
    
    def search(
        self,
        query: str,
        folder: str = "INBOX",
        limit: int = 10,
    ) -> List[Dict[str, Any]]:
        """Search emails by subject or sender.
        
        Args:
            query: Search query (searches subject and from)
            folder: IMAP folder to search
            limit: Maximum results
            
        Returns:
            List of matching emails
        """
        if not self.username or not self.password:
            return [{"error": "Email credentials not configured."}]
        
        try:
            import imaplib
            import email
            from email.header import decode_header
        except ImportError:
            return [{"error": "imaplib not available."}]
        
        try:
            mail = imaplib.IMAP4_SSL(self.imap_host, self.imap_port)
            mail.login(self.username, self.password)
            mail.select(folder)
            
            # Search by subject or from
            _, subject_nums = mail.search(None, f'SUBJECT "{query}"')
            _, from_nums = mail.search(None, f'FROM "{query}"')
            
            # Combine results
            all_nums = set(subject_nums[0].split() + from_nums[0].split())
            
            emails = []
            for num in list(all_nums)[-limit:]:
                _, msg_data = mail.fetch(num, "(RFC822)")
                
                for response_part in msg_data:
                    if isinstance(response_part, tuple):
                        msg = email.message_from_bytes(response_part[1])
                        
                        subject, encoding = decode_header(msg["Subject"])[0]
                        if isinstance(subject, bytes):
                            subject = subject.decode(encoding or "utf-8")
                        
                        from_addr, encoding = decode_header(msg.get("From", ""))[0]
                        if isinstance(from_addr, bytes):
                            from_addr = from_addr.decode(encoding or "utf-8")
                        
                        emails.append({
                            "subject": subject,
                            "from": from_addr,
                            "date": msg.get("Date", ""),
                        })
            
            mail.close()
            mail.logout()
            
            return emails
            
        except Exception as e:
            logger.error(f"Email search error: {e}")
            return [{"error": str(e)}]


# Convenience functions for direct use
def send_email(
    to: str,
    subject: str,
    body: str,
    html: Optional[str] = None,
    cc: Optional[str] = None,
    bcc: Optional[str] = None,
    attachments: Optional[List[str]] = None,
    provider: str = "gmail",
) -> str:
    """Send an email using environment credentials.
    
    Args:
        to: Recipient email address
        subject: Email subject
        body: Email body text
        html: Optional HTML body
        cc: CC recipients
        bcc: BCC recipients
        attachments: List of file paths
        provider: Email provider (gmail, outlook, yahoo)
        
    Returns:
        Success or error message
    """
    tool = EmailTool(provider=provider)
    return tool.send(to=to, subject=subject, body=body, html=html, cc=cc, bcc=bcc, attachments=attachments)


def read_emails(
    folder: str = "INBOX",
    limit: int = 10,
    unread_only: bool = False,
    provider: str = "gmail",
) -> List[Dict[str, Any]]:
    """Read emails from inbox using environment credentials.
    
    Args:
        folder: IMAP folder
        limit: Max emails to return
        unread_only: Only unread emails
        provider: Email provider
        
    Returns:
        List of email dictionaries
    """
    tool = EmailTool(provider=provider)
    return tool.read(folder=folder, limit=limit, unread_only=unread_only)


def search_emails(
    query: str,
    folder: str = "INBOX",
    limit: int = 10,
    provider: str = "gmail",
) -> List[Dict[str, Any]]:
    """Search emails using environment credentials.
    
    Args:
        query: Search query
        folder: IMAP folder
        limit: Max results
        provider: Email provider
        
    Returns:
        List of matching emails
    """
    tool = EmailTool(provider=provider)
    return tool.search(query=query, folder=folder, limit=limit)



"""Google Maps Tool for PraisonAI Agents.

Geocoding, places, and directions using Google Maps API.

Usage:
    from praisonai_tools import GoogleMapsTool
    
    maps = GoogleMapsTool()
    location = maps.geocode("1600 Amphitheatre Parkway, Mountain View, CA")

Environment Variables:
    GOOGLE_MAPS_API_KEY: Google Maps API key
"""

import os
import logging
from typing import Any, Dict, List, Optional, Union

from praisonai_tools.tools.base import BaseTool

logger = logging.getLogger(__name__)


class GoogleMapsTool(BaseTool):
    """Tool for Google Maps operations."""
    
    name = "google_maps"
    description = "Geocode addresses, search places, and get directions."
    
    def __init__(self, api_key: Optional[str] = None):
        self.api_key = api_key or os.getenv("GOOGLE_MAPS_API_KEY")
        self._client = None
        super().__init__()
    
    @property
    def client(self):
        if self._client is None:
            try:
                import googlemaps
            except ImportError:
                raise ImportError("googlemaps not installed. Install with: pip install googlemaps")
            
            if not self.api_key:
                raise ValueError("GOOGLE_MAPS_API_KEY required")
            
            self._client = googlemaps.Client(key=self.api_key)
        return self._client
    
    def run(
        self,
        action: str = "geocode",
        address: Optional[str] = None,
        place_id: Optional[str] = None,
        query: Optional[str] = None,
        origin: Optional[str] = None,
        destination: Optional[str] = None,
        **kwargs
    ) -> Union[str, Dict[str, Any], List[Dict[str, Any]]]:
        action = action.lower().replace("-", "_")
        
        if action == "geocode":
            return self.geocode(address=address)
        elif action == "reverse_geocode":
            return self.reverse_geocode(lat=kwargs.get("lat"), lng=kwargs.get("lng"))
        elif action == "search_places":
            return self.search_places(query=query, **kwargs)
        elif action == "place_details":
            return self.place_details(place_id=place_id)
        elif action == "directions":
            return self.directions(origin=origin, destination=destination, **kwargs)
        elif action == "distance":
            return self.distance(origin=origin, destination=destination)
        else:
            return {"error": f"Unknown action: {action}"}
    
    def geocode(self, address: str) -> Dict[str, Any]:
        """Geocode an address."""
        if not address:
            return {"error": "address is required"}
        
        try:
            results = self.client.geocode(address)
            if not results:
                return {"error": "No results found"}
            
            result = results[0]
            return {
                "formatted_address": result.get("formatted_address"),
                "lat": result["geometry"]["location"]["lat"],
                "lng": result["geometry"]["location"]["lng"],
                "place_id": result.get("place_id"),
            }
        except Exception as e:
            logger.error(f"Google Maps geocode error: {e}")
            return {"error": str(e)}
    
    def reverse_geocode(self, lat: float, lng: float) -> Dict[str, Any]:
        """Reverse geocode coordinates."""
        if lat is None or lng is None:
            return {"error": "lat and lng are required"}
        
        try:
            results = self.client.reverse_geocode((lat, lng))
            if not results:
                return {"error": "No results found"}
            
            result = results[0]
            return {
                "formatted_address": result.get("formatted_address"),
                "place_id": result.get("place_id"),
            }
        except Exception as e:
            logger.error(f"Google Maps reverse_geocode error: {e}")
            return {"error": str(e)}
    
    def search_places(self, query: str, location: Optional[str] = None, radius: int = 5000) -> List[Dict[str, Any]]:
        """Search for places."""
        if not query:
            return [{"error": "query is required"}]
        
        try:
            params = {"query": query}
            if location:
                geo = self.geocode(location)
                if "error" not in geo:
                    params["location"] = (geo["lat"], geo["lng"])
                    params["radius"] = radius
            
            results = self.client.places(**params)
            
            places = []
            for place in results.get("results", [])[:10]:
                places.append({
                    "name": place.get("name"),
                    "address": place.get("formatted_address"),
                    "place_id": place.get("place_id"),
                    "rating": place.get("rating"),
                    "lat": place.get("geometry", {}).get("location", {}).get("lat"),
                    "lng": place.get("geometry", {}).get("location", {}).get("lng"),
                })
            return places
        except Exception as e:
            logger.error(f"Google Maps search_places error: {e}")
            return [{"error": str(e)}]
    
    def place_details(self, place_id: str) -> Dict[str, Any]:
        """Get place details."""
        if not place_id:
            return {"error": "place_id is required"}
        
        try:
            result = self.client.place(place_id)
            place = result.get("result", {})
            
            return {
                "name": place.get("name"),
                "address": place.get("formatted_address"),
                "phone": place.get("formatted_phone_number"),
                "website": place.get("website"),
                "rating": place.get("rating"),
                "reviews": place.get("user_ratings_total"),
                "hours": place.get("opening_hours", {}).get("weekday_text"),
            }
        except Exception as e:
            logger.error(f"Google Maps place_details error: {e}")
            return {"error": str(e)}
    
    def directions(self, origin: str, destination: str, mode: str = "driving") -> Dict[str, Any]:
        """Get directions."""
        if not origin or not destination:
            return {"error": "origin and destination are required"}
        
        try:
            results = self.client.directions(origin, destination, mode=mode)
            if not results:
                return {"error": "No route found"}
            
            route = results[0]
            leg = route["legs"][0]
            
            return {
                "distance": leg["distance"]["text"],
                "duration": leg["duration"]["text"],
                "start_address": leg["start_address"],
                "end_address": leg["end_address"],
                "steps": [s["html_instructions"] for s in leg["steps"][:10]],
            }
        except Exception as e:
            logger.error(f"Google Maps directions error: {e}")
            return {"error": str(e)}
    
    def distance(self, origin: str, destination: str) -> Dict[str, Any]:
        """Get distance between two points."""
        if not origin or not destination:
            return {"error": "origin and destination are required"}
        
        try:
            result = self.client.distance_matrix(origin, destination)
            element = result["rows"][0]["elements"][0]
            
            if element["status"] != "OK":
                return {"error": element["status"]}
            
            return {
                "distance": element["distance"]["text"],
                "duration": element["duration"]["text"],
            }
        except Exception as e:
            logger.error(f"Google Maps distance error: {e}")
            return {"error": str(e)}


def geocode_address(address: str) -> Dict[str, Any]:
    """Geocode address."""
    return GoogleMapsTool().geocode(address=address)



"""
Recipe Tools - Shared tools for Agent-Recipes digital input→output workflows.

This module provides 12 core tools:
1. media_tool - Audio/video operations via ffmpeg
2. doc_tool - PDF/document operations via poppler/pandoc
3. image_tool - Image operations via Pillow/ImageMagick
4. data_tool - CSV/JSON/Parquet operations via pandas
5. repo_tool - Git operations
6. web_tool - URL fetching and content extraction
7. archive_tool - Zip/tar operations
8. whisper_tool - Audio transcription via OpenAI Whisper API
9. llm_tool - Unified LLM interface for all recipes
10. vision_tool - Image analysis, captioning, tagging
11. chart_tool - Chart/visualization generation
12. email_tool - Email parsing and extraction

All tools follow the pattern:
- Structured dataclass return types
- check_dependencies() method
- Safe defaults
- Logging via standard logger
- Clear exceptions with actionable messages
"""

from .base import RecipeToolBase, RecipeToolResult, DependencyError
from .media_tool import MediaTool, MediaProbeResult, media_probe, media_extract_audio, media_normalize, media_trim, media_extract_frames
from .doc_tool import DocTool, DocProbeResult, doc_to_markdown, doc_to_pdf, doc_extract_text, doc_extract_images
from .image_tool import ImageTool, ImageProbeResult, image_optimize, image_resize, image_thumbnail, image_montage
from .data_tool import DataTool, DataProfileResult, data_profile, data_clean, data_convert, data_infer_schema
from .repo_tool import RepoTool, RepoInfo, repo_log, repo_diff, repo_files, repo_info
from .web_tool import WebTool, WebContent, web_fetch, web_extract_article, web_fetch_sitemap
from .archive_tool import ArchiveTool, ArchiveManifest, archive_create, archive_extract, archive_list
from .whisper_tool import WhisperTool, TranscriptResult, whisper_transcribe
from .llm_tool import LLMTool, LLMResponse, LLMMessage, llm_complete, llm_extract_json
from .vision_tool import VisionTool, VisionResult, vision_caption, vision_tag, vision_ocr
from .chart_tool import ChartTool, ChartResult, chart_bar, chart_line, chart_pie
from .email_tool import EmailTool, ParsedEmail, EmailAttachment, ExtractedData, email_parse, email_extract

__all__ = [
    # Base
    "RecipeToolBase",
    "RecipeToolResult",
    "DependencyError",
    # Media
    "MediaTool",
    "MediaProbeResult",
    "media_probe",
    "media_extract_audio",
    "media_normalize",
    "media_trim",
    "media_extract_frames",
    # Doc
    "DocTool",
    "DocProbeResult",
    "doc_to_markdown",
    "doc_to_pdf",
    "doc_extract_text",
    "doc_extract_images",
    # Image
    "ImageTool",
    "ImageProbeResult",
    "image_optimize",
    "image_resize",
    "image_thumbnail",
    "image_montage",
    # Data
    "DataTool",
    "DataProfileResult",
    "data_profile",
    "data_clean",
    "data_convert",
    "data_infer_schema",
    # Repo
    "RepoTool",
    "RepoInfo",
    "repo_log",
    "repo_diff",
    "repo_files",
    "repo_info",
    # Web
    "WebTool",
    "WebContent",
    "web_fetch",
    "web_extract_article",
    "web_fetch_sitemap",
    # Archive
    "ArchiveTool",
    "ArchiveManifest",
    "archive_create",
    "archive_extract",
    "archive_list",
    # Whisper
    "WhisperTool",
    "TranscriptResult",
    "whisper_transcribe",
    # LLM
    "LLMTool",
    "LLMResponse",
    "LLMMessage",
    "llm_complete",
    "llm_extract_json",
    # Vision
    "VisionTool",
    "VisionResult",
    "vision_caption",
    "vision_tag",
    "vision_ocr",
    # Chart
    "ChartTool",
    "ChartResult",
    "chart_bar",
    "chart_line",
    "chart_pie",
    # Email
    "EmailTool",
    "ParsedEmail",
    "EmailAttachment",
    "ExtractedData",
    "email_parse",
    "email_extract",
]



"""Tests for RepoTool."""

import subprocess
import pytest

from praisonai_tools.recipe_tools.repo_tool import RepoTool, repo_info, repo_log


class TestRepoTool:
    """Unit tests for RepoTool."""
    
    @pytest.fixture
    def tool(self):
        return RepoTool(verbose=True)
    
    @pytest.fixture
    def git_repo(self, temp_dir):
        """Create a temporary git repository."""
        repo_dir = temp_dir / "test_repo"
        repo_dir.mkdir()
        
        # Initialize git repo
        subprocess.run(["git", "init"], cwd=repo_dir, capture_output=True)
        subprocess.run(["git", "config", "user.email", "test@test.com"], cwd=repo_dir, capture_output=True)
        subprocess.run(["git", "config", "user.name", "Test User"], cwd=repo_dir, capture_output=True)
        
        # Create some files and commits
        (repo_dir / "file1.txt").write_text("content 1")
        subprocess.run(["git", "add", "."], cwd=repo_dir, capture_output=True)
        subprocess.run(["git", "commit", "-m", "feat: initial commit"], cwd=repo_dir, capture_output=True)
        
        (repo_dir / "file2.txt").write_text("content 2")
        subprocess.run(["git", "add", "."], cwd=repo_dir, capture_output=True)
        subprocess.run(["git", "commit", "-m", "fix: add file2"], cwd=repo_dir, capture_output=True)
        
        return repo_dir
    
    @pytest.mark.unit
    def test_check_dependencies(self, tool):
        """Test dependency checking."""
        deps = tool.check_dependencies()
        assert "git" in deps
    
    @pytest.mark.requires_git
    def test_info(self, tool, git_repo, has_git):
        """Test getting repo info."""
        if not has_git:
            pytest.skip("git not available")
        
        result = tool.info(git_repo)
        
        assert result.path == str(git_repo)
        assert result.name == "test_repo"
        assert result.commit_count >= 2
    
    @pytest.mark.requires_git
    def test_log(self, tool, git_repo, has_git):
        """Test getting commit log."""
        if not has_git:
            pytest.skip("git not available")
        
        commits = tool.log(git_repo, limit=10)
        
        assert len(commits) >= 2
        assert commits[0].message  # Most recent commit
    
    @pytest.mark.requires_git
    def test_diff(self, tool, git_repo, has_git):
        """Test getting diff."""
        if not has_git:
            pytest.skip("git not available")
        
        diff = tool.diff(git_repo, "HEAD~1", "HEAD")
        
        assert "file2.txt" in diff
    
    @pytest.mark.requires_git
    def test_files(self, tool, git_repo, has_git):
        """Test listing files."""
        if not has_git:
            pytest.skip("git not available")
        
        files = tool.files(git_repo)
        
        assert "file1.txt" in files
        assert "file2.txt" in files
    
    @pytest.mark.requires_git
    def test_categorize_commits(self, tool, git_repo, has_git):
        """Test commit categorization."""
        if not has_git:
            pytest.skip("git not available")
        
        commits = tool.log(git_repo)
        categories = tool.categorize_commits(commits)
        
        assert "feat" in categories or "fix" in categories
    
    @pytest.mark.unit
    def test_not_a_repo(self, tool, temp_dir):
        """Test error on non-repo directory."""
        with pytest.raises(ValueError, match="Not a git repository"):
            tool.info(temp_dir)


class TestRepoToolConvenienceFunctions:
    """Test convenience functions."""
    
    @pytest.fixture
    def git_repo(self, temp_dir):
        """Create a temporary git repository."""
        repo_dir = temp_dir / "test_repo"
        repo_dir.mkdir()
        subprocess.run(["git", "init"], cwd=repo_dir, capture_output=True)
        subprocess.run(["git", "config", "user.email", "test@test.com"], cwd=repo_dir, capture_output=True)
        subprocess.run(["git", "config", "user.name", "Test User"], cwd=repo_dir, capture_output=True)
        (repo_dir / "file.txt").write_text("content")
        subprocess.run(["git", "add", "."], cwd=repo_dir, capture_output=True)
        subprocess.run(["git", "commit", "-m", "initial"], cwd=repo_dir, capture_output=True)
        return repo_dir
    
    @pytest.mark.requires_git
    def test_repo_info(self, git_repo, has_git):
        """Test repo_info function."""
        if not has_git:
            pytest.skip("git not available")
        
        result = repo_info(git_repo)
        assert result.commit_count >= 1
    
    @pytest.mark.requires_git
    def test_repo_log(self, git_repo, has_git):
        """Test repo_log function."""
        if not has_git:
            pytest.skip("git not available")
        
        commits = repo_log(git_repo)
        assert len(commits) >= 1



"""
Vision Tool - Image analysis, captioning, and tagging with LLM vision models.

Provides capabilities for:
- Image captioning and alt-text generation
- Object detection and tagging
- OCR and text extraction from images
- Image-based Q&A
"""

import os
import base64
import logging
from dataclasses import dataclass, field
from typing import Optional, List, Dict, Any
from pathlib import Path

try:
    from .base import RecipeToolBase, RecipeToolResult
except ImportError:
    from base import RecipeToolBase, RecipeToolResult

logger = logging.getLogger(__name__)


@dataclass
class VisionResult(RecipeToolResult):
    """Result from vision analysis."""
    description: str = ""
    tags: List[str] = field(default_factory=list)
    objects: List[Dict[str, Any]] = field(default_factory=list)
    text_content: str = ""
    confidence: float = 0.0
    model: str = ""
    provider: str = ""


class VisionTool(RecipeToolBase):
    """Vision analysis tool using LLM vision models."""
    
    SUPPORTED_PROVIDERS = ["openai", "anthropic", "google"]
    SUPPORTED_FORMATS = [".jpg", ".jpeg", ".png", ".gif", ".webp", ".bmp"]
    
    def __init__(
        self,
        provider: str = "openai",
        model: Optional[str] = None,
        api_key: Optional[str] = None,
        redact_pii: bool = True,
    ):
        self.provider = provider.lower()
        self.model = model or self._default_model()
        self.api_key = api_key or self._get_api_key()
        self.redact_pii = redact_pii
        self._client = None
    
    def _default_model(self) -> str:
        """Get default vision model for provider."""
        defaults = {
            "openai": "gpt-4o-mini",
            "anthropic": "claude-3-haiku-20240307",
            "google": "gemini-1.5-flash",
        }
        return defaults.get(self.provider, "gpt-4o-mini")
    
    def _get_api_key(self) -> Optional[str]:
        """Get API key from environment."""
        env_vars = {
            "openai": "OPENAI_API_KEY",
            "anthropic": "ANTHROPIC_API_KEY",
            "google": "GOOGLE_API_KEY",
        }
        env_var = env_vars.get(self.provider)
        return os.environ.get(env_var) if env_var else None
    
    def check_dependencies(self) -> Dict[str, bool]:
        """Check if required dependencies are available."""
        deps = {}
        
        if self.provider == "openai":
            try:
                import openai
                deps["openai"] = True
            except ImportError:
                deps["openai"] = False
        elif self.provider == "anthropic":
            try:
                import anthropic
                deps["anthropic"] = True
            except ImportError:
                deps["anthropic"] = False
        elif self.provider == "google":
            try:
                import google.generativeai
                deps["google-generativeai"] = True
            except ImportError:
                deps["google-generativeai"] = False
        
        deps["api_key"] = bool(self.api_key)
        return deps
    
    def _encode_image(self, image_path: str) -> str:
        """Encode image to base64."""
        with open(image_path, "rb") as f:
            return base64.b64encode(f.read()).decode("utf-8")
    
    def _get_mime_type(self, image_path: str) -> str:
        """Get MIME type from file extension."""
        ext = Path(image_path).suffix.lower()
        mime_types = {
            ".jpg": "image/jpeg",
            ".jpeg": "image/jpeg",
            ".png": "image/png",
            ".gif": "image/gif",
            ".webp": "image/webp",
            ".bmp": "image/bmp",
        }
        return mime_types.get(ext, "image/jpeg")
    
    def _get_client(self):
        """Get or create the API client."""
        if self._client is not None:
            return self._client
        
        if self.provider == "openai":
            import openai
            self._client = openai.OpenAI(api_key=self.api_key)
        elif self.provider == "anthropic":
            import anthropic
            self._client = anthropic.Anthropic(api_key=self.api_key)
        elif self.provider == "google":
            import google.generativeai as genai
            genai.configure(api_key=self.api_key)
            self._client = genai.GenerativeModel(self.model)
        
        return self._client
    
    def analyze(
        self,
        image_path: str,
        prompt: Optional[str] = None,
        include_tags: bool = True,
        include_objects: bool = False,
        include_text: bool = False,
    ) -> VisionResult:
        """
        Analyze an image with vision model.
        
        Args:
            image_path: Path to image file
            prompt: Custom prompt for analysis
            include_tags: Include keyword tags
            include_objects: Include object detection
            include_text: Include OCR text extraction
            
        Returns:
            VisionResult with analysis
        """
        if not os.path.exists(image_path):
            return VisionResult(success=False, error=f"Image not found: {image_path}")
        
        ext = Path(image_path).suffix.lower()
        if ext not in self.SUPPORTED_FORMATS:
            return VisionResult(success=False, error=f"Unsupported format: {ext}")
        
        # Build analysis prompt
        if prompt:
            analysis_prompt = prompt
        else:
            parts = ["Describe this image in detail."]
            if include_tags:
                parts.append("List relevant tags/keywords.")
            if include_objects:
                parts.append("Identify and list all objects visible.")
            if include_text:
                parts.append("Extract any text visible in the image.")
            analysis_prompt = " ".join(parts)
        
        if self.redact_pii:
            analysis_prompt += " Do not include any personally identifiable information (names, addresses, phone numbers, etc.) in your response."
        
        client = self._get_client()
        
        if self.provider == "openai":
            return self._analyze_openai(client, image_path, analysis_prompt, include_tags, include_text)
        elif self.provider == "anthropic":
            return self._analyze_anthropic(client, image_path, analysis_prompt, include_tags, include_text)
        elif self.provider == "google":
            return self._analyze_google(client, image_path, analysis_prompt, include_tags, include_text)
        else:
            return VisionResult(success=False, error=f"Unsupported provider: {self.provider}")
    
    def _analyze_openai(
        self,
        client,
        image_path: str,
        prompt: str,
        include_tags: bool,
        include_text: bool,
    ) -> VisionResult:
        """Analyze using OpenAI Vision."""
        base64_image = self._encode_image(image_path)
        mime_type = self._get_mime_type(image_path)
        
        response = client.chat.completions.create(
            model=self.model,
            messages=[
                {
                    "role": "user",
                    "content": [
                        {"type": "text", "text": prompt},
                        {
                            "type": "image_url",
                            "image_url": {
                                "url": f"data:{mime_type};base64,{base64_image}"
                            }
                        }
                    ]
                }
            ],
            max_tokens=1000,
        )
        
        content = response.choices[0].message.content
        
        # Parse tags if requested
        tags = []
        if include_tags and "tags" in prompt.lower():
            import re
            tag_match = re.search(r'tags?[:\s]+([^\n]+)', content, re.IGNORECASE)
            if tag_match:
                tags = [t.strip() for t in tag_match.group(1).split(',')]
        
        return VisionResult(
            success=True,
            description=content,
            tags=tags,
            model=self.model,
            provider=self.provider,
        )
    
    def _analyze_anthropic(
        self,
        client,
        image_path: str,
        prompt: str,
        include_tags: bool,
        include_text: bool,
    ) -> VisionResult:
        """Analyze using Anthropic Vision."""
        base64_image = self._encode_image(image_path)
        mime_type = self._get_mime_type(image_path)
        
        response = client.messages.create(
            model=self.model,
            max_tokens=1000,
            messages=[
                {
                    "role": "user",
                    "content": [
                        {
                            "type": "image",
                            "source": {
                                "type": "base64",
                                "media_type": mime_type,
                                "data": base64_image,
                            }
                        },
                        {"type": "text", "text": prompt}
                    ]
                }
            ]
        )
        
        content = response.content[0].text
        
        return VisionResult(
            success=True,
            description=content,
            model=self.model,
            provider=self.provider,
        )
    
    def _analyze_google(
        self,
        client,
        image_path: str,
        prompt: str,
        include_tags: bool,
        include_text: bool,
    ) -> VisionResult:
        """Analyze using Google Vision."""
        import PIL.Image
        
        image = PIL.Image.open(image_path)
        response = client.generate_content([prompt, image])
        
        return VisionResult(
            success=True,
            description=response.text,
            model=self.model,
            provider=self.provider,
        )
    
    def caption(self, image_path: str, style: str = "descriptive") -> str:
        """
        Generate a caption for an image.
        
        Args:
            image_path: Path to image
            style: Caption style - descriptive, concise, alt-text, social
            
        Returns:
            Caption string
        """
        prompts = {
            "descriptive": "Describe this image in 2-3 sentences.",
            "concise": "Describe this image in one sentence.",
            "alt-text": "Write an accessibility alt-text description for this image. Be concise but descriptive.",
            "social": "Write an engaging social media caption for this image.",
        }
        
        prompt = prompts.get(style, prompts["descriptive"])
        result = self.analyze(image_path, prompt=prompt, include_tags=False)
        return result.description if result.success else ""
    
    def tag(self, image_path: str, max_tags: int = 10) -> List[str]:
        """
        Generate tags/keywords for an image.
        
        Args:
            image_path: Path to image
            max_tags: Maximum number of tags
            
        Returns:
            List of tags
        """
        prompt = f"List up to {max_tags} relevant tags/keywords for this image, separated by commas. Output only the tags, nothing else."
        result = self.analyze(image_path, prompt=prompt, include_tags=True)
        
        if result.success:
            # Parse comma-separated tags from response
            tags = [t.strip() for t in result.description.split(',')]
            return tags[:max_tags]
        return []
    
    def extract_text(self, image_path: str) -> str:
        """
        Extract text from an image (OCR).
        
        Args:
            image_path: Path to image
            
        Returns:
            Extracted text
        """
        prompt = "Extract and transcribe all text visible in this image. Output only the extracted text, preserving the layout as much as possible."
        result = self.analyze(image_path, prompt=prompt, include_text=True)
        return result.description if result.success else ""
    
    def ask(self, image_path: str, question: str) -> str:
        """
        Ask a question about an image.
        
        Args:
            image_path: Path to image
            question: Question to ask
            
        Returns:
            Answer string
        """
        result = self.analyze(image_path, prompt=question)
        return result.description if result.success else ""


# Convenience functions
def vision_caption(image_path: str, style: str = "descriptive", provider: str = "openai") -> str:
    """Generate caption for image."""
    tool = VisionTool(provider=provider)
    return tool.caption(image_path, style)


def vision_tag(image_path: str, max_tags: int = 10, provider: str = "openai") -> List[str]:
    """Generate tags for image."""
    tool = VisionTool(provider=provider)
    return tool.tag(image_path, max_tags)


def vision_ocr(image_path: str, provider: str = "openai") -> str:
    """Extract text from image."""
    tool = VisionTool(provider=provider)
    return tool.extract_text(image_path)


__all__ = [
    "VisionTool",
    "VisionResult",
    "vision_caption",
    "vision_tag",
    "vision_ocr",
]



"""
LLM Tool - Unified LLM interface for all recipes.

Provides a consistent interface for interacting with various LLM providers
(OpenAI, Anthropic, Google, etc.) with streaming, JSON schema outputs,
and prompt versioning support.
"""

import os
import json
import logging
from dataclasses import dataclass, field
from typing import Optional, List, Dict, Any, Union, Generator

try:
    from .base import RecipeToolBase, RecipeToolResult, DependencyError
except ImportError:
    from base import RecipeToolBase, RecipeToolResult, DependencyError

logger = logging.getLogger(__name__)


@dataclass
class LLMResponse(RecipeToolResult):
    """Result from LLM completion."""
    content: str = ""
    model: str = ""
    provider: str = ""
    usage: Dict[str, int] = field(default_factory=dict)
    finish_reason: str = ""
    raw_response: Optional[Dict[str, Any]] = None


@dataclass
class LLMMessage:
    """A message in a conversation."""
    role: str  # system, user, assistant
    content: str


class LLMTool(RecipeToolBase):
    """Unified LLM interface supporting multiple providers."""
    
    SUPPORTED_PROVIDERS = ["openai", "anthropic", "google", "ollama"]
    
    def __init__(
        self,
        provider: str = "openai",
        model: Optional[str] = None,
        api_key: Optional[str] = None,
        base_url: Optional[str] = None,
        temperature: float = 0.7,
        max_tokens: int = 4096,
    ):
        self.provider = provider.lower()
        self.model = model or self._default_model()
        self.api_key = api_key or self._get_api_key()
        self.base_url = base_url
        self.temperature = temperature
        self.max_tokens = max_tokens
        self._client = None
    
    def _default_model(self) -> str:
        """Get default model for provider."""
        defaults = {
            "openai": "gpt-4o-mini",
            "anthropic": "claude-3-haiku-20240307",
            "google": "gemini-1.5-flash",
            "ollama": "llama3.2",
        }
        return defaults.get(self.provider, "gpt-4o-mini")
    
    def _get_api_key(self) -> Optional[str]:
        """Get API key from environment."""
        env_vars = {
            "openai": "OPENAI_API_KEY",
            "anthropic": "ANTHROPIC_API_KEY",
            "google": "GOOGLE_API_KEY",
        }
        env_var = env_vars.get(self.provider)
        return os.environ.get(env_var) if env_var else None
    
    def check_dependencies(self) -> Dict[str, bool]:
        """Check if required dependencies are available."""
        deps = {}
        
        if self.provider == "openai":
            try:
                import openai
                deps["openai"] = True
            except ImportError:
                deps["openai"] = False
        
        elif self.provider == "anthropic":
            try:
                import anthropic
                deps["anthropic"] = True
            except ImportError:
                deps["anthropic"] = False
        
        elif self.provider == "google":
            try:
                import google.generativeai
                deps["google-generativeai"] = True
            except ImportError:
                deps["google-generativeai"] = False
        
        # Check API key
        deps["api_key"] = bool(self.api_key) or self.provider == "ollama"
        
        return deps
    
    def _get_client(self):
        """Get or create the LLM client."""
        if self._client is not None:
            return self._client
        
        if self.provider == "openai":
            import openai
            self._client = openai.OpenAI(
                api_key=self.api_key,
                base_url=self.base_url,
            )
        elif self.provider == "anthropic":
            import anthropic
            self._client = anthropic.Anthropic(api_key=self.api_key)
        elif self.provider == "google":
            import google.generativeai as genai
            genai.configure(api_key=self.api_key)
            self._client = genai.GenerativeModel(self.model)
        elif self.provider == "ollama":
            import openai
            self._client = openai.OpenAI(
                api_key="ollama",
                base_url=self.base_url or "http://localhost:11434/v1",
            )
        
        return self._client
    
    def complete(
        self,
        prompt: str,
        system: Optional[str] = None,
        messages: Optional[List[LLMMessage]] = None,
        temperature: Optional[float] = None,
        max_tokens: Optional[int] = None,
        json_schema: Optional[Dict[str, Any]] = None,
        seed: Optional[int] = None,
    ) -> LLMResponse:
        """
        Generate a completion from the LLM.
        
        Args:
            prompt: The user prompt
            system: Optional system message
            messages: Optional conversation history
            temperature: Override default temperature
            max_tokens: Override default max tokens
            json_schema: Optional JSON schema for structured output
            seed: Optional seed for reproducibility
            
        Returns:
            LLMResponse with the completion
        """
        temp = temperature if temperature is not None else self.temperature
        max_tok = max_tokens if max_tokens is not None else self.max_tokens
        
        # Build messages
        msg_list = []
        if system:
            msg_list.append({"role": "system", "content": system})
        if messages:
            for m in messages:
                msg_list.append({"role": m.role, "content": m.content})
        msg_list.append({"role": "user", "content": prompt})
        
        client = self._get_client()
        
        if self.provider in ["openai", "ollama"]:
            return self._complete_openai(client, msg_list, temp, max_tok, json_schema, seed)
        elif self.provider == "anthropic":
            return self._complete_anthropic(client, msg_list, temp, max_tok, system)
        elif self.provider == "google":
            return self._complete_google(client, prompt, temp, max_tok)
        else:
            raise ValueError(f"Unsupported provider: {self.provider}")
    
    def _complete_openai(
        self,
        client,
        messages: List[Dict],
        temperature: float,
        max_tokens: int,
        json_schema: Optional[Dict] = None,
        seed: Optional[int] = None,
    ) -> LLMResponse:
        """Complete using OpenAI-compatible API."""
        kwargs = {
            "model": self.model,
            "messages": messages,
            "temperature": temperature,
            "max_tokens": max_tokens,
        }
        
        if seed is not None:
            kwargs["seed"] = seed
        
        if json_schema:
            kwargs["response_format"] = {"type": "json_object"}
        
        response = client.chat.completions.create(**kwargs)
        
        return LLMResponse(
            success=True,
            content=response.choices[0].message.content,
            model=response.model,
            provider=self.provider,
            usage={
                "prompt_tokens": response.usage.prompt_tokens,
                "completion_tokens": response.usage.completion_tokens,
                "total_tokens": response.usage.total_tokens,
            },
            finish_reason=response.choices[0].finish_reason,
        )
    
    def _complete_anthropic(
        self,
        client,
        messages: List[Dict],
        temperature: float,
        max_tokens: int,
        system: Optional[str] = None,
    ) -> LLMResponse:
        """Complete using Anthropic API."""
        # Filter out system message and use it separately
        filtered_messages = [m for m in messages if m["role"] != "system"]
        
        kwargs = {
            "model": self.model,
            "messages": filtered_messages,
            "temperature": temperature,
            "max_tokens": max_tokens,
        }
        
        if system:
            kwargs["system"] = system
        
        response = client.messages.create(**kwargs)
        
        return LLMResponse(
            success=True,
            content=response.content[0].text,
            model=response.model,
            provider=self.provider,
            usage={
                "prompt_tokens": response.usage.input_tokens,
                "completion_tokens": response.usage.output_tokens,
                "total_tokens": response.usage.input_tokens + response.usage.output_tokens,
            },
            finish_reason=response.stop_reason,
        )
    
    def _complete_google(
        self,
        client,
        prompt: str,
        temperature: float,
        max_tokens: int,
    ) -> LLMResponse:
        """Complete using Google Generative AI."""
        generation_config = {
            "temperature": temperature,
            "max_output_tokens": max_tokens,
        }
        
        response = client.generate_content(
            prompt,
            generation_config=generation_config,
        )
        
        return LLMResponse(
            success=True,
            content=response.text,
            model=self.model,
            provider=self.provider,
            usage={},
            finish_reason="stop",
        )
    
    def stream(
        self,
        prompt: str,
        system: Optional[str] = None,
        temperature: Optional[float] = None,
        max_tokens: Optional[int] = None,
    ) -> Generator[str, None, None]:
        """
        Stream a completion from the LLM.
        
        Yields chunks of the response as they arrive.
        """
        temp = temperature if temperature is not None else self.temperature
        max_tok = max_tokens if max_tokens is not None else self.max_tokens
        
        messages = []
        if system:
            messages.append({"role": "system", "content": system})
        messages.append({"role": "user", "content": prompt})
        
        client = self._get_client()
        
        if self.provider in ["openai", "ollama"]:
            response = client.chat.completions.create(
                model=self.model,
                messages=messages,
                temperature=temp,
                max_tokens=max_tok,
                stream=True,
            )
            for chunk in response:
                if chunk.choices[0].delta.content:
                    yield chunk.choices[0].delta.content
        
        elif self.provider == "anthropic":
            with client.messages.stream(
                model=self.model,
                messages=messages,
                temperature=temp,
                max_tokens=max_tok,
            ) as stream:
                for text in stream.text_stream:
                    yield text
        
        elif self.provider == "google":
            response = client.generate_content(
                prompt,
                generation_config={"temperature": temp, "max_output_tokens": max_tok},
                stream=True,
            )
            for chunk in response:
                if chunk.text:
                    yield chunk.text
    
    def extract_json(
        self,
        prompt: str,
        schema: Dict[str, Any],
        system: Optional[str] = None,
    ) -> Dict[str, Any]:
        """
        Extract structured JSON from LLM response.
        
        Args:
            prompt: The prompt describing what to extract
            schema: JSON schema describing expected output
            system: Optional system message
            
        Returns:
            Parsed JSON dictionary
        """
        schema_str = json.dumps(schema, indent=2)
        enhanced_prompt = f"""{prompt}

Respond with valid JSON matching this schema:
{schema_str}

Output only the JSON, no other text."""
        
        response = self.complete(
            prompt=enhanced_prompt,
            system=system or "You are a helpful assistant that outputs valid JSON.",
            json_schema=schema,
        )
        
        try:
            return json.loads(response.content)
        except json.JSONDecodeError:
            # Try to extract JSON from response
            import re
            json_match = re.search(r'\{[\s\S]*\}', response.content)
            if json_match:
                return json.loads(json_match.group())
            raise ValueError(f"Could not parse JSON from response: {response.content[:200]}")


# Convenience functions
def llm_complete(
    prompt: str,
    provider: str = "openai",
    model: Optional[str] = None,
    system: Optional[str] = None,
    temperature: float = 0.7,
    max_tokens: int = 4096,
) -> str:
    """Quick completion with default settings."""
    tool = LLMTool(provider=provider, model=model, temperature=temperature, max_tokens=max_tokens)
    response = tool.complete(prompt, system=system)
    return response.content


def llm_extract_json(
    prompt: str,
    schema: Dict[str, Any],
    provider: str = "openai",
    model: Optional[str] = None,
) -> Dict[str, Any]:
    """Quick JSON extraction."""
    tool = LLMTool(provider=provider, model=model)
    return tool.extract_json(prompt, schema)


__all__ = [
    "LLMTool",
    "LLMResponse",
    "LLMMessage",
    "llm_complete",
    "llm_extract_json",
]



"""Tools package for PraisonAI Tools.

This module provides base classes and ready-to-use tools for AI agents.

Base classes for custom tools:
- BaseTool, @tool decorator, FunctionTool

Ready-to-use tools:
- EmailTool: Send/read emails via SMTP/IMAP
- SlackTool: Send messages to Slack
- DiscordTool: Send messages to Discord
- GitHubTool: Interact with GitHub repos/issues
- ImageTool: Generate images with DALL-E
- WeatherTool: Get weather data
- YouTubeTool: Search YouTube, get transcripts
- TTSTool: Text-to-speech conversion
"""

from praisonai_tools.tools.base import BaseTool, ToolResult, ToolValidationError, validate_tool
from praisonai_tools.tools.decorator import tool, FunctionTool, is_tool, get_tool_schema

# Lazy imports for tools to avoid loading dependencies until needed
def __getattr__(name):
    """Lazy load tool classes."""
    tool_map = {
        # Email
        "EmailTool": "email_tool",
        "send_email": "email_tool",
        "read_emails": "email_tool",
        "search_emails": "email_tool",
        # Slack
        "SlackTool": "slack_tool",
        "send_slack_message": "slack_tool",
        "get_slack_channels": "slack_tool",
        "get_slack_history": "slack_tool",
        # Discord
        "DiscordTool": "discord_tool",
        "send_discord_webhook": "discord_tool",
        "send_discord_message": "discord_tool",
        # GitHub
        "GitHubTool": "github_tool",
        "search_github_repos": "github_tool",
        "get_github_repo": "github_tool",
        "create_github_issue": "github_tool",
        # Image Generation
        "ImageTool": "image_tool",
        "generate_image": "image_tool",
        # Weather
        "WeatherTool": "weather_tool",
        "get_weather": "weather_tool",
        "get_forecast": "weather_tool",
        "get_air_quality": "weather_tool",
        # YouTube
        "YouTubeTool": "youtube_tool",
        "search_youtube": "youtube_tool",
        "get_youtube_video": "youtube_tool",
        "get_youtube_transcript": "youtube_tool",
        # Text-to-Speech
        "TTSTool": "tts_tool",
        "text_to_speech": "tts_tool",
        "list_tts_voices": "tts_tool",
        # Telegram
        "TelegramTool": "telegram_tool",
        "send_telegram_message": "telegram_tool",
        # Notion
        "NotionTool": "notion_tool",
        "search_notion": "notion_tool",
        "create_notion_page": "notion_tool",
        # PostgreSQL
        "PostgresTool": "postgres_tool",
        "query_postgres": "postgres_tool",
        "list_postgres_tables": "postgres_tool",
        # Reddit
        "RedditTool": "reddit_tool",
        "search_reddit": "reddit_tool",
        "get_reddit_hot": "reddit_tool",
        # Docker
        "DockerTool": "docker_tool",
        "list_docker_containers": "docker_tool",
        "run_docker_container": "docker_tool",
        # MySQL
        "MySQLTool": "mysql_tool",
        "query_mysql": "mysql_tool",
        # SQLite
        "SQLiteTool": "sqlite_tool",
        "query_sqlite": "sqlite_tool",
        # MongoDB
        "MongoDBTool": "mongodb_tool",
        "query_mongodb": "mongodb_tool",
        # Redis
        "RedisTool": "redis_tool",
        "redis_get": "redis_tool",
        "redis_set": "redis_tool",
        # DuckDuckGo
        "DuckDuckGoTool": "duckduckgo_tool",
        "duckduckgo_search": "duckduckgo_tool",
        # Tavily
        "TavilyTool": "tavily_tool",
        "tavily_search": "tavily_tool",
        # Wikipedia
        "WikipediaTool": "wikipedia_tool",
        "wikipedia_search": "wikipedia_tool",
        # ArXiv
        "ArxivTool": "arxiv_tool",
        "arxiv_search": "arxiv_tool",
        # Calculator
        "CalculatorTool": "calculator_tool",
        "calculate": "calculator_tool",
        # Shell
        "ShellTool": "shell_tool",
        "shell_execute": "shell_tool",
        # File
        "FileTool": "file_tool",
        "read_file": "file_tool",
        "write_file": "file_tool",
        # Firecrawl
        "FirecrawlTool": "firecrawl_tool",
        "firecrawl_scrape": "firecrawl_tool",
        # Serper
        "SerperTool": "serper_tool",
        "serper_search": "serper_tool",
        # Jina
        "JinaTool": "jina_tool",
        "jina_read": "jina_tool",
        # Google Calendar
        "GoogleCalendarTool": "google_calendar_tool",
        "list_calendar_events": "google_calendar_tool",
        # Google Sheets
        "GoogleSheetsTool": "google_sheets_tool",
        "read_google_sheet": "google_sheets_tool",
        # Jira
        "JiraTool": "jira_tool",
        "jira_search": "jira_tool",
        # Trello
        "TrelloTool": "trello_tool",
        "list_trello_boards": "trello_tool",
        # YFinance
        "YFinanceTool": "yfinance_tool",
        "get_stock_price": "yfinance_tool",
        # Pinecone
        "PineconeTool": "pinecone_tool",
        "pinecone_query": "pinecone_tool",
        # Qdrant
        "QdrantTool": "qdrant_tool",
        "qdrant_search": "qdrant_tool",
        # Chroma
        "ChromaTool": "chroma_tool",
        "chroma_query": "chroma_tool",
        # HackerNews
        "HackerNewsTool": "hackernews_tool",
        "get_hackernews_top": "hackernews_tool",
        # Twilio
        "TwilioTool": "twilio_tool",
        "send_sms": "twilio_tool",
        # Spotify
        "SpotifyTool": "spotify_tool",
        "spotify_search": "spotify_tool",
        # Linear
        "LinearTool": "linear_tool",
        "list_linear_issues": "linear_tool",
        # Crawl4AI
        "Crawl4AITool": "crawl4ai_tool",
        "crawl4ai_crawl": "crawl4ai_tool",
        # Exa
        "ExaTool": "exa_tool",
        "exa_search": "exa_tool",
        # DuckDB
        "DuckDBTool": "duckdb_tool",
        "duckdb_query": "duckdb_tool",
        # Neo4j
        "Neo4jTool": "neo4j_tool",
        "neo4j_query": "neo4j_tool",
        # Weaviate
        "WeaviateTool": "weaviate_tool",
        "weaviate_search": "weaviate_tool",
        # LanceDB
        "LanceDBTool": "lancedb_tool",
        "lancedb_search": "lancedb_tool",
        # Supabase
        "SupabaseTool": "supabase_tool",
        "supabase_select": "supabase_tool",
        # Milvus
        "MilvusTool": "milvus_tool",
        "milvus_search": "milvus_tool",
        # Google Maps
        "GoogleMapsTool": "google_maps_tool",
        "geocode_address": "google_maps_tool",
        # Zendesk
        "ZendeskTool": "zendesk_tool",
        "list_zendesk_tickets": "zendesk_tool",
        # Shopify
        "ShopifyTool": "shopify_tool",
        "list_shopify_products": "shopify_tool",
        # Brave Search
        "BraveSearchTool": "brave_search_tool",
        "brave_search": "brave_search_tool",
        # PubMed
        "PubMedTool": "pubmed_tool",
        "pubmed_search": "pubmed_tool",
        # SerpAPI
        "SerpAPITool": "serpapi_tool",
        "serpapi_search": "serpapi_tool",
        # SearxNG
        "SearxNGTool": "searxng_tool",
        "searxng_search": "searxng_tool",
        # LinkUp
        "LinkUpTool": "linkup_tool",
        "linkup_search": "linkup_tool",
        # Baidu Search
        "BaiduSearchTool": "baidu_search_tool",
        "baidu_search": "baidu_search_tool",
        # Valyu
        "ValyuTool": "valyu_tool",
        "valyu_search": "valyu_tool",
        # Gmail
        "GmailTool": "gmail_tool",
        "list_gmail_emails": "gmail_tool",
        # WhatsApp
        "WhatsAppTool": "whatsapp_tool",
        "send_whatsapp_message": "whatsapp_tool",
        # Zoom
        "ZoomTool": "zoom_tool",
        "list_zoom_meetings": "zoom_tool",
        # Webex
        "WebexTool": "webex_tool",
        "list_webex_rooms": "webex_tool",
        # X (Twitter)
        "XTool": "x_tool",
        "post_to_x": "x_tool",
        # Newspaper
        "NewspaperTool": "newspaper_tool",
        "extract_article": "newspaper_tool",
        # Trafilatura
        "TrafilaturaTool": "trafilatura_tool",
        "trafilatura_extract": "trafilatura_tool",
        # Spider
        "SpiderTool": "spider_tool",
        "spider_crawl": "spider_tool",
        # BrowserBase
        "BrowserBaseTool": "browserbase_tool",
        "browserbase_scrape": "browserbase_tool",
        # BrightData
        "BrightDataTool": "brightdata_tool",
        "brightdata_scrape": "brightdata_tool",
        # Oxylabs
        "OxylabsTool": "oxylabs_tool",
        "oxylabs_scrape": "oxylabs_tool",
        # ScrapeGraph
        "ScrapeGraphTool": "scrapegraph_tool",
        "scrapegraph_scrape": "scrapegraph_tool",
        # AgentQL
        "AgentQLTool": "agentql_tool",
        "agentql_query": "agentql_tool",
        # DynamoDB
        "DynamoDBTool": "dynamodb_tool",
        "dynamodb_scan": "dynamodb_tool",
        # Firestore
        "FirestoreTool": "firestore_tool",
        "firestore_list": "firestore_tool",
        # GCS
        "GCSTool": "gcs_tool",
        "gcs_list_objects": "gcs_tool",
        # BigQuery
        "BigQueryTool": "bigquery_tool",
        "bigquery_query": "bigquery_tool",
        # SingleStore
        "SingleStoreTool": "singlestore_tool",
        "singlestore_query": "singlestore_tool",
        # SurrealDB
        "SurrealDBTool": "surrealdb_tool",
        "surrealdb_query": "surrealdb_tool",
        # CSV
        "CSVTool": "csv_tool",
        "read_csv": "csv_tool",
        # Pandas
        "PandasTool": "pandas_tool",
        "pandas_read_csv": "pandas_tool",
        # Redshift
        "RedshiftTool": "redshift_tool",
        "redshift_query": "redshift_tool",
        # ElevenLabs
        "ElevenLabsTool": "elevenlabs_tool",
        "elevenlabs_speak": "elevenlabs_tool",
        # Replicate
        "ReplicateTool": "replicate_tool",
        "replicate_run": "replicate_tool",
        # Fal
        "FalTool": "fal_tool",
        "fal_run": "fal_tool",
        # Giphy
        "GiphyTool": "giphy_tool",
        "giphy_search": "giphy_tool",
        # Apify
        "ApifyTool": "apify_tool",
        "apify_run_actor": "apify_tool",
        # Airflow
        "AirflowTool": "airflow_tool",
        "list_airflow_dags": "airflow_tool",
        # AWS Lambda
        "AWSLambdaTool": "aws_lambda_tool",
        "invoke_lambda": "aws_lambda_tool",
        # AWS SES
        "AWSSESTool": "aws_ses_tool",
        "ses_send_email": "aws_ses_tool",
        # Resend
        "ResendTool": "resend_tool",
        "resend_send_email": "resend_tool",
        # ClickUp
        "ClickUpTool": "clickup_tool",
        "clickup_list_tasks": "clickup_tool",
        # Confluence
        "ConfluenceTool": "confluence_tool",
        "confluence_search": "confluence_tool",
        # Todoist
        "TodoistTool": "todoist_tool",
        "todoist_list_tasks": "todoist_tool",
        # Bitbucket
        "BitbucketTool": "bitbucket_tool",
        "bitbucket_list_repos": "bitbucket_tool",
        # OpenBB
        "OpenBBTool": "openbb_tool",
        "openbb_stock": "openbb_tool",
        # E2B
        "E2BTool": "e2b_tool",
        "e2b_run_code": "e2b_tool",
        # Sleep
        "SleepTool": "sleep_tool",
        "sleep_wait": "sleep_tool",
        # Python
        "PythonTool": "python_tool",
        "python_execute": "python_tool",
        # Mem0
        "Mem0Tool": "mem0_tool",
        "mem0_add": "mem0_tool",
        # PGVector
        "PGVectorTool": "pgvector_tool",
        "pgvector_search": "pgvector_tool",
        # Cassandra
        "CassandraTool": "cassandra_tool",
        "cassandra_query": "cassandra_tool",
        # ClickHouse
        "ClickHouseTool": "clickhouse_tool",
        "clickhouse_query": "clickhouse_tool",
        # Upstash
        "UpstashTool": "upstash_tool",
        "upstash_get": "upstash_tool",
        # Couchbase
        "CouchbaseTool": "couchbase_tool",
        "couchbase_get": "couchbase_tool",
        # Google Drive
        "GoogleDriveTool": "google_drive_tool",
        "list_drive_files": "google_drive_tool",
        # LumaLabs
        "LumaLabsTool": "lumalabs_tool",
        "lumalabs_generate": "lumalabs_tool",
        # OpenCV
        "OpenCVTool": "opencv_tool",
        "opencv_resize": "opencv_tool",
        # CalCom
        "CalComTool": "calcom_tool",
        "calcom_list_bookings": "calcom_tool",
        # Cartesia
        "CartesiaTool": "cartesia_tool",
        "cartesia_speak": "cartesia_tool",
        # Brandfetch
        "BrandfetchTool": "brandfetch_tool",
        "brandfetch_get_brand": "brandfetch_tool",
        # Daytona
        "DaytonaTool": "daytona_tool",
        "daytona_list_workspaces": "daytona_tool",
        # Composio
        "ComposioTool": "composio_tool",
        "composio_execute": "composio_tool",
        # JSON
        "JSONTool": "json_tool",
        "read_json": "json_tool",
        # Zep
        "ZepTool": "zep_tool",
        "zep_add_memory": "zep_tool",
        # WebBrowser
        "WebBrowserTool": "webbrowser_tool",
        "webbrowser_get_page": "webbrowser_tool",
        # WebTools
        "WebToolsTool": "webtools_tool",
        "webtools_fetch": "webtools_tool",
        # LocalFileSystem
        "LocalFileSystemTool": "local_filesystem_tool",
        "list_local_directory": "local_filesystem_tool",
        # Knowledge
        "KnowledgeTool": "knowledge_tool",
        "knowledge_search": "knowledge_tool",
        # Visualization
        "VisualizationTool": "visualization_tool",
        "create_bar_chart": "visualization_tool",
    }
    
    if name in tool_map:
        module_name = tool_map[name]
        from importlib import import_module
        module = import_module(f".{module_name}", __package__)
        return getattr(module, name)
    
    raise AttributeError(f"module {__name__!r} has no attribute {name!r}")

__all__ = [
    # Base classes
    "BaseTool",
    "ToolResult",
    "ToolValidationError",
    "validate_tool",
    # Decorator
    "tool",
    "FunctionTool",
    "is_tool",
    "get_tool_schema",
    # Email Tool
    "EmailTool",
    "send_email",
    "read_emails",
    "search_emails",
    # Slack Tool
    "SlackTool",
    "send_slack_message",
    "get_slack_channels",
    "get_slack_history",
    # Discord Tool
    "DiscordTool",
    "send_discord_webhook",
    "send_discord_message",
    # GitHub Tool
    "GitHubTool",
    "search_github_repos",
    "get_github_repo",
    "create_github_issue",
    # Image Tool
    "ImageTool",
    "generate_image",
    # Weather Tool
    "WeatherTool",
    "get_weather",
    "get_forecast",
    "get_air_quality",
    # YouTube Tool
    "YouTubeTool",
    "search_youtube",
    "get_youtube_video",
    "get_youtube_transcript",
    # TTS Tool
    "TTSTool",
    "text_to_speech",
    "list_tts_voices",
    # Telegram Tool
    "TelegramTool",
    "send_telegram_message",
    # Notion Tool
    "NotionTool",
    "search_notion",
    "create_notion_page",
    # PostgreSQL Tool
    "PostgresTool",
    "query_postgres",
    "list_postgres_tables",
    # Reddit Tool
    "RedditTool",
    "search_reddit",
    "get_reddit_hot",
    # Docker Tool
    "DockerTool",
    "list_docker_containers",
    "run_docker_container",
    # MySQL Tool
    "MySQLTool",
    "query_mysql",
    # SQLite Tool
    "SQLiteTool",
    "query_sqlite",
    # MongoDB Tool
    "MongoDBTool",
    "query_mongodb",
    # Redis Tool
    "RedisTool",
    "redis_get",
    "redis_set",
    # DuckDuckGo Tool
    "DuckDuckGoTool",
    "duckduckgo_search",
    # Tavily Tool
    "TavilyTool",
    "tavily_search",
    # Wikipedia Tool
    "WikipediaTool",
    "wikipedia_search",
    # ArXiv Tool
    "ArxivTool",
    "arxiv_search",
    # Calculator Tool
    "CalculatorTool",
    "calculate",
    # Shell Tool
    "ShellTool",
    "shell_execute",
    # File Tool
    "FileTool",
    "read_file",
    "write_file",
    # Firecrawl Tool
    "FirecrawlTool",
    "firecrawl_scrape",
    # Serper Tool
    "SerperTool",
    "serper_search",
    # Jina Tool
    "JinaTool",
    "jina_read",
    # Google Calendar Tool
    "GoogleCalendarTool",
    "list_calendar_events",
    # Google Sheets Tool
    "GoogleSheetsTool",
    "read_google_sheet",
    # Jira Tool
    "JiraTool",
    "jira_search",
    # Trello Tool
    "TrelloTool",
    "list_trello_boards",
    # YFinance Tool
    "YFinanceTool",
    "get_stock_price",
    # Pinecone Tool
    "PineconeTool",
    "pinecone_query",
    # Qdrant Tool
    "QdrantTool",
    "qdrant_search",
    # Chroma Tool
    "ChromaTool",
    "chroma_query",
    # HackerNews Tool
    "HackerNewsTool",
    "get_hackernews_top",
    # Twilio Tool
    "TwilioTool",
    "send_sms",
    # Spotify Tool
    "SpotifyTool",
    "spotify_search",
    # Linear Tool
    "LinearTool",
    "list_linear_issues",
    # Crawl4AI Tool
    "Crawl4AITool",
    "crawl4ai_crawl",
    # Exa Tool
    "ExaTool",
    "exa_search",
    # DuckDB Tool
    "DuckDBTool",
    "duckdb_query",
    # Neo4j Tool
    "Neo4jTool",
    "neo4j_query",
    # Weaviate Tool
    "WeaviateTool",
    "weaviate_search",
    # LanceDB Tool
    "LanceDBTool",
    "lancedb_search",
    # Supabase Tool
    "SupabaseTool",
    "supabase_select",
    # Milvus Tool
    "MilvusTool",
    "milvus_search",
    # Google Maps Tool
    "GoogleMapsTool",
    "geocode_address",
    # Zendesk Tool
    "ZendeskTool",
    "list_zendesk_tickets",
    # Shopify Tool
    "ShopifyTool",
    "list_shopify_products",
]



"""Zoom Tool for PraisonAI Agents.

Manage Zoom meetings.

Usage:
    from praisonai_tools import ZoomTool
    
    zoom = ZoomTool()
    meetings = zoom.list_meetings()

Environment Variables:
    ZOOM_ACCOUNT_ID: Zoom account ID
    ZOOM_CLIENT_ID: Zoom client ID
    ZOOM_CLIENT_SECRET: Zoom client secret
"""

import os
import logging
from typing import Any, Dict, List, Optional, Union

from praisonai_tools.tools.base import BaseTool

logger = logging.getLogger(__name__)


class ZoomTool(BaseTool):
    """Tool for Zoom meetings."""
    
    name = "zoom"
    description = "Manage Zoom meetings."
    
    def __init__(
        self,
        account_id: Optional[str] = None,
        client_id: Optional[str] = None,
        client_secret: Optional[str] = None,
    ):
        self.account_id = account_id or os.getenv("ZOOM_ACCOUNT_ID")
        self.client_id = client_id or os.getenv("ZOOM_CLIENT_ID")
        self.client_secret = client_secret or os.getenv("ZOOM_CLIENT_SECRET")
        self._token = None
        super().__init__()
    
    def _get_token(self) -> str:
        if self._token:
            return self._token
        
        import requests
        import base64
        
        credentials = base64.b64encode(f"{self.client_id}:{self.client_secret}".encode()).decode()
        resp = requests.post(
            "https://zoom.us/oauth/token",
            headers={"Authorization": f"Basic {credentials}"},
            data={"grant_type": "account_credentials", "account_id": self.account_id},
            timeout=10,
        )
        self._token = resp.json().get("access_token")
        return self._token
    
    def run(
        self,
        action: str = "list",
        **kwargs
    ) -> Union[str, Dict[str, Any], List[Dict[str, Any]]]:
        action = action.lower().replace("-", "_")
        
        if action == "list":
            return self.list_meetings()
        elif action == "create":
            return self.create_meeting(**kwargs)
        elif action == "get":
            return self.get_meeting(meeting_id=kwargs.get("meeting_id"))
        elif action == "delete":
            return self.delete_meeting(meeting_id=kwargs.get("meeting_id"))
        else:
            return {"error": f"Unknown action: {action}"}
    
    def list_meetings(self) -> List[Dict[str, Any]]:
        """List meetings."""
        if not all([self.account_id, self.client_id, self.client_secret]):
            return [{"error": "ZOOM_ACCOUNT_ID, ZOOM_CLIENT_ID, ZOOM_CLIENT_SECRET required"}]
        
        try:
            import requests
        except ImportError:
            return [{"error": "requests not installed"}]
        
        try:
            token = self._get_token()
            headers = {"Authorization": f"Bearer {token}"}
            resp = requests.get("https://api.zoom.us/v2/users/me/meetings", headers=headers, timeout=10)
            data = resp.json()
            
            meetings = []
            for m in data.get("meetings", []):
                meetings.append({
                    "id": m.get("id"),
                    "topic": m.get("topic"),
                    "start_time": m.get("start_time"),
                    "duration": m.get("duration"),
                    "join_url": m.get("join_url"),
                })
            return meetings
        except Exception as e:
            logger.error(f"Zoom list error: {e}")
            return [{"error": str(e)}]
    
    def create_meeting(self, topic: str, duration: int = 60, start_time: str = None) -> Dict[str, Any]:
        """Create meeting."""
        if not topic:
            return {"error": "topic is required"}
        if not all([self.account_id, self.client_id, self.client_secret]):
            return {"error": "Zoom credentials required"}
        
        try:
            import requests
        except ImportError:
            return {"error": "requests not installed"}
        
        try:
            token = self._get_token()
            headers = {"Authorization": f"Bearer {token}", "Content-Type": "application/json"}
            data = {"topic": topic, "type": 2, "duration": duration}
            if start_time:
                data["start_time"] = start_time
            
            resp = requests.post(
                "https://api.zoom.us/v2/users/me/meetings",
                headers=headers,
                json=data,
                timeout=10,
            )
            result = resp.json()
            
            return {
                "id": result.get("id"),
                "topic": result.get("topic"),
                "join_url": result.get("join_url"),
                "start_url": result.get("start_url"),
            }
        except Exception as e:
            logger.error(f"Zoom create error: {e}")
            return {"error": str(e)}
    
    def get_meeting(self, meeting_id: str) -> Dict[str, Any]:
        """Get meeting details."""
        if not meeting_id:
            return {"error": "meeting_id is required"}
        
        try:
            import requests
        except ImportError:
            return {"error": "requests not installed"}
        
        try:
            token = self._get_token()
            headers = {"Authorization": f"Bearer {token}"}
            resp = requests.get(f"https://api.zoom.us/v2/meetings/{meeting_id}", headers=headers, timeout=10)
            return resp.json()
        except Exception as e:
            logger.error(f"Zoom get error: {e}")
            return {"error": str(e)}
    
    def delete_meeting(self, meeting_id: str) -> Dict[str, Any]:
        """Delete meeting."""
        if not meeting_id:
            return {"error": "meeting_id is required"}
        
        try:
            import requests
        except ImportError:
            return {"error": "requests not installed"}
        
        try:
            token = self._get_token()
            headers = {"Authorization": f"Bearer {token}"}
            resp = requests.delete(f"https://api.zoom.us/v2/meetings/{meeting_id}", headers=headers, timeout=10)
            if resp.status_code == 204:
                return {"success": True}
            return resp.json()
        except Exception as e:
            logger.error(f"Zoom delete error: {e}")
            return {"error": str(e)}


def list_zoom_meetings() -> List[Dict[str, Any]]:
    """List Zoom meetings."""
    return ZoomTool().list_meetings()



"""Reddit Tool for PraisonAI Agents.

Search Reddit, get posts, and interact with subreddits.

Usage:
    from praisonai_tools import RedditTool
    
    reddit = RedditTool()  # Uses REDDIT_CLIENT_ID, REDDIT_CLIENT_SECRET env vars
    
    # Search posts
    posts = reddit.search("python programming", subreddit="learnpython")
    
    # Get hot posts
    hot = reddit.get_hot(subreddit="technology", limit=10)

Environment Variables:
    REDDIT_CLIENT_ID: Reddit API client ID
    REDDIT_CLIENT_SECRET: Reddit API client secret
    REDDIT_USER_AGENT: User agent string (optional)
"""

import os
import logging
from typing import Any, Dict, List, Optional, Union

from praisonai_tools.tools.base import BaseTool

logger = logging.getLogger(__name__)


class RedditTool(BaseTool):
    """Tool for interacting with Reddit."""
    
    name = "reddit"
    description = "Search Reddit, get posts from subreddits, and read comments."
    
    def __init__(
        self,
        client_id: Optional[str] = None,
        client_secret: Optional[str] = None,
        user_agent: Optional[str] = None,
    ):
        """Initialize RedditTool.
        
        Args:
            client_id: Reddit API client ID
            client_secret: Reddit API client secret
            user_agent: User agent string
        """
        self.client_id = client_id or os.getenv("REDDIT_CLIENT_ID")
        self.client_secret = client_secret or os.getenv("REDDIT_CLIENT_SECRET")
        self.user_agent = user_agent or os.getenv("REDDIT_USER_AGENT", "PraisonAI/1.0")
        self._reddit = None
        super().__init__()
    
    @property
    def reddit(self):
        """Lazy-load Reddit client."""
        if self._reddit is None:
            try:
                import praw
            except ImportError:
                raise ImportError("praw not installed. Install with: pip install praw")
            
            if not self.client_id or not self.client_secret:
                raise ValueError("REDDIT_CLIENT_ID and REDDIT_CLIENT_SECRET required")
            
            self._reddit = praw.Reddit(
                client_id=self.client_id,
                client_secret=self.client_secret,
                user_agent=self.user_agent,
            )
        return self._reddit
    
    def run(
        self,
        action: str = "search",
        query: Optional[str] = None,
        subreddit: Optional[str] = None,
        post_id: Optional[str] = None,
        limit: int = 10,
        **kwargs
    ) -> Union[str, Dict[str, Any], List[Dict[str, Any]]]:
        """Execute Reddit action."""
        action = action.lower().replace("-", "_")
        
        if action == "search":
            return self.search(query=query, subreddit=subreddit, limit=limit)
        elif action == "hot":
            return self.get_hot(subreddit=subreddit, limit=limit)
        elif action == "top":
            return self.get_top(subreddit=subreddit, limit=limit, **kwargs)
        elif action == "new":
            return self.get_new(subreddit=subreddit, limit=limit)
        elif action == "get_post":
            return self.get_post(post_id=post_id)
        elif action == "get_comments":
            return self.get_comments(post_id=post_id, limit=limit)
        elif action == "subreddit_info":
            return self.get_subreddit_info(subreddit=subreddit)
        else:
            return {"error": f"Unknown action: {action}"}
    
    def search(
        self,
        query: str,
        subreddit: Optional[str] = None,
        sort: str = "relevance",
        time_filter: str = "all",
        limit: int = 10,
    ) -> List[Dict[str, Any]]:
        """Search Reddit posts.
        
        Args:
            query: Search query
            subreddit: Limit to specific subreddit
            sort: "relevance", "hot", "top", "new", "comments"
            time_filter: "all", "day", "week", "month", "year"
            limit: Max results
            
        Returns:
            List of posts
        """
        if not query:
            return [{"error": "query is required"}]
        
        try:
            if subreddit:
                sub = self.reddit.subreddit(subreddit)
                results = sub.search(query, sort=sort, time_filter=time_filter, limit=limit)
            else:
                results = self.reddit.subreddit("all").search(query, sort=sort, time_filter=time_filter, limit=limit)
            
            posts = []
            for post in results:
                posts.append(self._format_post(post))
            
            return posts
        except Exception as e:
            logger.error(f"Reddit search error: {e}")
            return [{"error": str(e)}]
    
    def get_hot(self, subreddit: str = "all", limit: int = 10) -> List[Dict[str, Any]]:
        """Get hot posts from a subreddit.
        
        Args:
            subreddit: Subreddit name
            limit: Max posts
            
        Returns:
            List of posts
        """
        try:
            sub = self.reddit.subreddit(subreddit)
            posts = []
            for post in sub.hot(limit=limit):
                posts.append(self._format_post(post))
            return posts
        except Exception as e:
            logger.error(f"Reddit get_hot error: {e}")
            return [{"error": str(e)}]
    
    def get_top(
        self,
        subreddit: str = "all",
        time_filter: str = "day",
        limit: int = 10,
    ) -> List[Dict[str, Any]]:
        """Get top posts from a subreddit.
        
        Args:
            subreddit: Subreddit name
            time_filter: "hour", "day", "week", "month", "year", "all"
            limit: Max posts
            
        Returns:
            List of posts
        """
        try:
            sub = self.reddit.subreddit(subreddit)
            posts = []
            for post in sub.top(time_filter=time_filter, limit=limit):
                posts.append(self._format_post(post))
            return posts
        except Exception as e:
            logger.error(f"Reddit get_top error: {e}")
            return [{"error": str(e)}]
    
    def get_new(self, subreddit: str = "all", limit: int = 10) -> List[Dict[str, Any]]:
        """Get new posts from a subreddit.
        
        Args:
            subreddit: Subreddit name
            limit: Max posts
            
        Returns:
            List of posts
        """
        try:
            sub = self.reddit.subreddit(subreddit)
            posts = []
            for post in sub.new(limit=limit):
                posts.append(self._format_post(post))
            return posts
        except Exception as e:
            logger.error(f"Reddit get_new error: {e}")
            return [{"error": str(e)}]
    
    def get_post(self, post_id: str) -> Dict[str, Any]:
        """Get a specific post by ID.
        
        Args:
            post_id: Reddit post ID
            
        Returns:
            Post details
        """
        if not post_id:
            return {"error": "post_id is required"}
        
        try:
            post = self.reddit.submission(id=post_id)
            return self._format_post(post, include_body=True)
        except Exception as e:
            logger.error(f"Reddit get_post error: {e}")
            return {"error": str(e)}
    
    def get_comments(self, post_id: str, limit: int = 20) -> List[Dict[str, Any]]:
        """Get comments from a post.
        
        Args:
            post_id: Reddit post ID
            limit: Max comments
            
        Returns:
            List of comments
        """
        if not post_id:
            return [{"error": "post_id is required"}]
        
        try:
            post = self.reddit.submission(id=post_id)
            post.comments.replace_more(limit=0)
            
            comments = []
            for comment in post.comments[:limit]:
                comments.append({
                    "id": comment.id,
                    "author": str(comment.author) if comment.author else "[deleted]",
                    "body": comment.body[:500],
                    "score": comment.score,
                    "created_utc": comment.created_utc,
                })
            
            return comments
        except Exception as e:
            logger.error(f"Reddit get_comments error: {e}")
            return [{"error": str(e)}]
    
    def get_subreddit_info(self, subreddit: str) -> Dict[str, Any]:
        """Get subreddit information.
        
        Args:
            subreddit: Subreddit name
            
        Returns:
            Subreddit info
        """
        if not subreddit:
            return {"error": "subreddit is required"}
        
        try:
            sub = self.reddit.subreddit(subreddit)
            return {
                "name": sub.display_name,
                "title": sub.title,
                "description": sub.public_description[:500] if sub.public_description else "",
                "subscribers": sub.subscribers,
                "created_utc": sub.created_utc,
                "over18": sub.over18,
                "url": f"https://reddit.com{sub.url}",
            }
        except Exception as e:
            logger.error(f"Reddit get_subreddit_info error: {e}")
            return {"error": str(e)}
    
    def _format_post(self, post, include_body: bool = False) -> Dict[str, Any]:
        """Format a Reddit post."""
        result = {
            "id": post.id,
            "title": post.title,
            "author": str(post.author) if post.author else "[deleted]",
            "subreddit": post.subreddit.display_name,
            "score": post.score,
            "upvote_ratio": post.upvote_ratio,
            "num_comments": post.num_comments,
            "url": post.url,
            "permalink": f"https://reddit.com{post.permalink}",
            "created_utc": post.created_utc,
            "is_self": post.is_self,
        }
        
        if include_body and post.is_self:
            result["body"] = post.selftext[:2000] if post.selftext else ""
        
        return result


def search_reddit(query: str, subreddit: Optional[str] = None, limit: int = 10) -> List[Dict[str, Any]]:
    """Search Reddit posts."""
    return RedditTool().search(query=query, subreddit=subreddit, limit=limit)


def get_reddit_hot(subreddit: str = "all", limit: int = 10) -> List[Dict[str, Any]]:
    """Get hot Reddit posts."""
    return RedditTool().get_hot(subreddit=subreddit, limit=limit)



"""Pandas Tool for PraisonAI Agents.

DataFrame operations using Pandas.

Usage:
    from praisonai_tools import PandasTool
    
    pd_tool = PandasTool()
    data = pd_tool.read_csv("data.csv")
"""

import logging
from typing import Any, Dict, List, Optional, Union

from praisonai_tools.tools.base import BaseTool

logger = logging.getLogger(__name__)


class PandasTool(BaseTool):
    """Tool for Pandas DataFrame operations."""
    
    name = "pandas"
    description = "DataFrame operations using Pandas."
    
    def run(
        self,
        action: str = "read_csv",
        file_path: Optional[str] = None,
        **kwargs
    ) -> Union[str, Dict[str, Any], List[Dict[str, Any]]]:
        action = action.lower().replace("-", "_")
        
        if action == "read_csv":
            return self.read_csv(file_path=file_path, **kwargs)
        elif action == "read_excel":
            return self.read_excel(file_path=file_path, **kwargs)
        elif action == "read_json":
            return self.read_json(file_path=file_path)
        elif action == "describe":
            return self.describe(file_path=file_path)
        elif action == "query":
            return self.query(file_path=file_path, expr=kwargs.get("expr"))
        else:
            return {"error": f"Unknown action: {action}"}
    
    def read_csv(self, file_path: str, nrows: int = 100) -> List[Dict[str, Any]]:
        """Read CSV file."""
        if not file_path:
            return [{"error": "file_path is required"}]
        
        try:
            import pandas as pd
            df = pd.read_csv(file_path, nrows=nrows)
            return df.to_dict(orient="records")
        except ImportError:
            return [{"error": "pandas not installed"}]
        except Exception as e:
            logger.error(f"Pandas read_csv error: {e}")
            return [{"error": str(e)}]
    
    def read_excel(self, file_path: str, sheet_name: str = None, nrows: int = 100) -> List[Dict[str, Any]]:
        """Read Excel file."""
        if not file_path:
            return [{"error": "file_path is required"}]
        
        try:
            import pandas as pd
            df = pd.read_excel(file_path, sheet_name=sheet_name, nrows=nrows)
            return df.to_dict(orient="records")
        except ImportError:
            return [{"error": "pandas and openpyxl not installed"}]
        except Exception as e:
            logger.error(f"Pandas read_excel error: {e}")
            return [{"error": str(e)}]
    
    def read_json(self, file_path: str) -> List[Dict[str, Any]]:
        """Read JSON file."""
        if not file_path:
            return [{"error": "file_path is required"}]
        
        try:
            import pandas as pd
            df = pd.read_json(file_path)
            return df.to_dict(orient="records")
        except ImportError:
            return [{"error": "pandas not installed"}]
        except Exception as e:
            logger.error(f"Pandas read_json error: {e}")
            return [{"error": str(e)}]
    
    def describe(self, file_path: str) -> Dict[str, Any]:
        """Get DataFrame statistics."""
        if not file_path:
            return {"error": "file_path is required"}
        
        try:
            import pandas as pd
            df = pd.read_csv(file_path)
            return {
                "shape": df.shape,
                "columns": list(df.columns),
                "dtypes": df.dtypes.astype(str).to_dict(),
                "describe": df.describe().to_dict(),
            }
        except ImportError:
            return {"error": "pandas not installed"}
        except Exception as e:
            logger.error(f"Pandas describe error: {e}")
            return {"error": str(e)}
    
    def query(self, file_path: str, expr: str) -> List[Dict[str, Any]]:
        """Query DataFrame."""
        if not file_path or not expr:
            return [{"error": "file_path and expr are required"}]
        
        try:
            import pandas as pd
            df = pd.read_csv(file_path)
            result = df.query(expr)
            return result.to_dict(orient="records")
        except ImportError:
            return [{"error": "pandas not installed"}]
        except Exception as e:
            logger.error(f"Pandas query error: {e}")
            return [{"error": str(e)}]


def pandas_read_csv(file_path: str, nrows: int = 100) -> List[Dict[str, Any]]:
    """Read CSV with Pandas."""
    return PandasTool().read_csv(file_path=file_path, nrows=nrows)



"""
FCP Doctor - Health Checks for Final Cut Pro Integration

Provides diagnostic checks and remediation guidance for the FCP integration.
"""

from __future__ import annotations

import os
import platform
import shutil
from dataclasses import dataclass
from typing import Optional

from .commandpost import CommandPostBridge


@dataclass
class CheckResult:
    """Result of a health check."""
    name: str
    passed: bool
    message: str
    remediation: Optional[str] = None


class FCPDoctor:
    """
    Diagnostic tool for FCP integration health checks.
    
    Checks for:
    - macOS environment
    - Final Cut Pro installation
    - CommandPost installation
    - cmdpost CLI availability
    - Watch folder configuration
    """

    def __init__(self, commandpost: Optional[CommandPostBridge] = None):
        """
        Initialize the doctor.
        
        Args:
            commandpost: CommandPost bridge instance
        """
        self.commandpost = commandpost or CommandPostBridge()

    def run_all_checks(self) -> list[CheckResult]:
        """Run all health checks."""
        return [
            self.check_macos(),
            self.check_final_cut_pro(),
            self.check_commandpost_app(),
            self.check_cmdpost_cli(),
            self.check_watch_folder(),
            self.check_auto_import(),
            self.check_openai_api_key(),
        ]

    def check_macos(self) -> CheckResult:
        """Check if running on macOS."""
        is_macos = platform.system() == "Darwin"
        if is_macos:
            version = platform.mac_ver()[0]
            return CheckResult(
                name="macOS",
                passed=True,
                message=f"Running on macOS {version}",
            )
        else:
            return CheckResult(
                name="macOS",
                passed=False,
                message=f"Running on {platform.system()}, not macOS",
                remediation="FCP integration requires macOS. Other platforms can generate FCPXML but cannot auto-import.",
            )

    def check_final_cut_pro(self) -> CheckResult:
        """Check if Final Cut Pro is installed."""
        fcp_paths = [
            "/Applications/Final Cut Pro.app",
            os.path.expanduser("~/Applications/Final Cut Pro.app"),
        ]

        for path in fcp_paths:
            if os.path.isdir(path):
                return CheckResult(
                    name="Final Cut Pro",
                    passed=True,
                    message=f"Found at {path}",
                )

        return CheckResult(
            name="Final Cut Pro",
            passed=False,
            message="Final Cut Pro not found",
            remediation="Install Final Cut Pro from the Mac App Store.",
        )

    def check_commandpost_app(self) -> CheckResult:
        """Check if CommandPost app is installed."""
        cp_paths = [
            "/Applications/CommandPost.app",
            "/Applications/Utilities/CommandPost.app",
            os.path.expanduser("~/Applications/CommandPost.app"),
        ]

        for path in cp_paths:
            if os.path.isdir(path):
                return CheckResult(
                    name="CommandPost App",
                    passed=True,
                    message=f"Found at {path}",
                )

        return CheckResult(
            name="CommandPost App",
            passed=False,
            message="CommandPost not found",
            remediation="Download CommandPost from https://commandpost.io",
        )

    def check_cmdpost_cli(self) -> CheckResult:
        """Check if cmdpost CLI is available."""
        cmdpost_path = self.commandpost.cmdpost_path

        if cmdpost_path:
            return CheckResult(
                name="cmdpost CLI",
                passed=True,
                message=f"Found at {cmdpost_path}",
            )

        cmdpost_in_path = shutil.which("cmdpost")
        if cmdpost_in_path:
            return CheckResult(
                name="cmdpost CLI",
                passed=True,
                message=f"Found in PATH at {cmdpost_in_path}",
            )

        return CheckResult(
            name="cmdpost CLI",
            passed=False,
            message="cmdpost CLI not found",
            remediation=(
                "Ensure CommandPost is installed and running. "
                "The cmdpost CLI is bundled with CommandPost at: "
                "/Applications/CommandPost.app/Contents/Resources/extensions/hs/cmdpost"
            ),
        )

    def check_watch_folder(self) -> CheckResult:
        """Check if watch folder exists."""
        watch_folder = self.commandpost.watch_folder

        if os.path.isdir(watch_folder):
            return CheckResult(
                name="Watch Folder",
                passed=True,
                message=f"Exists at {watch_folder}",
            )

        return CheckResult(
            name="Watch Folder",
            passed=False,
            message=f"Watch folder not found: {watch_folder}",
            remediation=f"Run: mkdir -p {watch_folder}",
        )

    def check_auto_import(self) -> CheckResult:
        """Check if auto-import is enabled in CommandPost."""
        status = self.commandpost.get_status()

        if not status.cmdpost_available:
            return CheckResult(
                name="Auto-Import",
                passed=False,
                message="Cannot check - cmdpost CLI not available",
                remediation="Install and run CommandPost first.",
            )

        if status.auto_import_enabled:
            return CheckResult(
                name="Auto-Import",
                passed=True,
                message="Enabled in CommandPost",
            )

        return CheckResult(
            name="Auto-Import",
            passed=False,
            message="Auto-import not enabled or could not be verified",
            remediation=(
                "Run: praisonai-tools fcp bootstrap\n"
                "Or manually enable in CommandPost preferences."
            ),
        )

    def check_openai_api_key(self) -> CheckResult:
        """Check if OpenAI API key is configured."""
        api_key = os.environ.get("OPENAI_API_KEY")

        if api_key:
            masked = f"...{api_key[-4:]}" if len(api_key) > 4 else "****"
            return CheckResult(
                name="OpenAI API Key",
                passed=True,
                message=f"Set (ending in {masked})",
            )

        return CheckResult(
            name="OpenAI API Key",
            passed=False,
            message="OPENAI_API_KEY not set",
            remediation="Set with: export OPENAI_API_KEY=your-key",
        )

    def print_report(self, results: Optional[list[CheckResult]] = None) -> bool:
        """
        Print a formatted health check report.
        
        Args:
            results: Check results (runs all checks if not provided)
            
        Returns:
            True if all checks passed
        """
        if results is None:
            results = self.run_all_checks()

        print("\n" + "=" * 60)
        print("FCP Integration Health Check")
        print("=" * 60 + "\n")

        all_passed = True
        for result in results:
            status = "✓" if result.passed else "✗"
            print(f"  {status} {result.name}: {result.message}")
            if not result.passed:
                all_passed = False
                if result.remediation:
                    print(f"    → {result.remediation}")
            print()

        print("=" * 60)
        if all_passed:
            print("All checks passed! FCP integration is ready.")
        else:
            print("Some checks failed. See remediation steps above.")
        print("=" * 60 + "\n")

        return all_passed

    def get_summary(self, results: Optional[list[CheckResult]] = None) -> dict:
        """
        Get a summary of health check results.
        
        Args:
            results: Check results (runs all checks if not provided)
            
        Returns:
            Summary dictionary
        """
        if results is None:
            results = self.run_all_checks()

        passed = [r for r in results if r.passed]
        failed = [r for r in results if not r.passed]

        return {
            "total": len(results),
            "passed": len(passed),
            "failed": len(failed),
            "all_passed": len(failed) == 0,
            "checks": [
                {
                    "name": r.name,
                    "passed": r.passed,
                    "message": r.message,
                    "remediation": r.remediation,
                }
                for r in results
            ],
        }



"""Redis Tool for PraisonAI Agents.

Interact with Redis key-value store.

Usage:
    from praisonai_tools import RedisTool
    
    redis = RedisTool()
    redis.set(key="mykey", value="myvalue")
    value = redis.get(key="mykey")

Environment Variables:
    REDIS_URL: Redis connection URL (default: redis://localhost:6379)
    REDIS_HOST: Redis host
    REDIS_PORT: Redis port
    REDIS_PASSWORD: Redis password
"""

import os
import logging
from typing import Any, Dict, List, Optional, Union

from praisonai_tools.tools.base import BaseTool

logger = logging.getLogger(__name__)


class RedisTool(BaseTool):
    """Tool for interacting with Redis."""
    
    name = "redis"
    description = "Get, set, and manage Redis keys."
    
    def __init__(
        self,
        url: Optional[str] = None,
        host: Optional[str] = None,
        port: Optional[int] = None,
        password: Optional[str] = None,
        db: int = 0,
    ):
        self.url = url or os.getenv("REDIS_URL")
        self.host = host or os.getenv("REDIS_HOST", "localhost")
        self.port = port or int(os.getenv("REDIS_PORT", "6379"))
        self.password = password or os.getenv("REDIS_PASSWORD")
        self.db = db
        self._client = None
        super().__init__()
    
    @property
    def client(self):
        if self._client is None:
            try:
                import redis
            except ImportError:
                raise ImportError("redis not installed. Install with: pip install redis")
            
            if self.url:
                self._client = redis.from_url(self.url)
            else:
                self._client = redis.Redis(
                    host=self.host,
                    port=self.port,
                    password=self.password,
                    db=self.db,
                    decode_responses=True,
                )
        return self._client
    
    def run(
        self,
        action: str = "get",
        key: Optional[str] = None,
        value: Optional[str] = None,
        pattern: str = "*",
        **kwargs
    ) -> Union[str, Dict[str, Any], List[str]]:
        action = action.lower().replace("-", "_")
        
        if action == "get":
            return self.get(key=key)
        elif action == "set":
            return self.set(key=key, value=value, **kwargs)
        elif action == "delete":
            return self.delete(key=key)
        elif action == "keys":
            return self.keys(pattern=pattern)
        elif action == "exists":
            return self.exists(key=key)
        elif action == "expire":
            return self.expire(key=key, seconds=kwargs.get("seconds", 60))
        elif action == "ttl":
            return self.ttl(key=key)
        elif action == "hget":
            return self.hget(key=key, field=kwargs.get("field"))
        elif action == "hset":
            return self.hset(key=key, field=kwargs.get("field"), value=value)
        elif action == "hgetall":
            return self.hgetall(key=key)
        elif action == "lpush":
            return self.lpush(key=key, value=value)
        elif action == "lrange":
            return self.lrange(key=key, start=kwargs.get("start", 0), end=kwargs.get("end", -1))
        elif action == "info":
            return self.info()
        else:
            return {"error": f"Unknown action: {action}"}
    
    def get(self, key: str) -> Dict[str, Any]:
        """Get value by key."""
        if not key:
            return {"error": "key is required"}
        try:
            value = self.client.get(key)
            return {"key": key, "value": value}
        except Exception as e:
            return {"error": str(e)}
    
    def set(self, key: str, value: str, ex: Optional[int] = None) -> Dict[str, Any]:
        """Set key-value pair."""
        if not key:
            return {"error": "key is required"}
        if value is None:
            return {"error": "value is required"}
        try:
            self.client.set(key, value, ex=ex)
            return {"success": True, "key": key}
        except Exception as e:
            return {"error": str(e)}
    
    def delete(self, key: str) -> Dict[str, Any]:
        """Delete key."""
        if not key:
            return {"error": "key is required"}
        try:
            deleted = self.client.delete(key)
            return {"success": True, "deleted": deleted}
        except Exception as e:
            return {"error": str(e)}
    
    def keys(self, pattern: str = "*") -> List[str]:
        """List keys matching pattern."""
        try:
            return self.client.keys(pattern)
        except Exception as e:
            return [f"error: {e}"]
    
    def exists(self, key: str) -> Dict[str, Any]:
        """Check if key exists."""
        if not key:
            return {"error": "key is required"}
        try:
            exists = self.client.exists(key)
            return {"key": key, "exists": bool(exists)}
        except Exception as e:
            return {"error": str(e)}
    
    def expire(self, key: str, seconds: int) -> Dict[str, Any]:
        """Set key expiration."""
        if not key:
            return {"error": "key is required"}
        try:
            result = self.client.expire(key, seconds)
            return {"success": result, "key": key, "expires_in": seconds}
        except Exception as e:
            return {"error": str(e)}
    
    def ttl(self, key: str) -> Dict[str, Any]:
        """Get time to live."""
        if not key:
            return {"error": "key is required"}
        try:
            ttl = self.client.ttl(key)
            return {"key": key, "ttl": ttl}
        except Exception as e:
            return {"error": str(e)}
    
    def hget(self, key: str, field: str) -> Dict[str, Any]:
        """Get hash field."""
        if not key or not field:
            return {"error": "key and field are required"}
        try:
            value = self.client.hget(key, field)
            return {"key": key, "field": field, "value": value}
        except Exception as e:
            return {"error": str(e)}
    
    def hset(self, key: str, field: str, value: str) -> Dict[str, Any]:
        """Set hash field."""
        if not key or not field:
            return {"error": "key and field are required"}
        try:
            self.client.hset(key, field, value)
            return {"success": True, "key": key, "field": field}
        except Exception as e:
            return {"error": str(e)}
    
    def hgetall(self, key: str) -> Dict[str, Any]:
        """Get all hash fields."""
        if not key:
            return {"error": "key is required"}
        try:
            data = self.client.hgetall(key)
            return {"key": key, "data": data}
        except Exception as e:
            return {"error": str(e)}
    
    def lpush(self, key: str, value: str) -> Dict[str, Any]:
        """Push to list."""
        if not key:
            return {"error": "key is required"}
        try:
            length = self.client.lpush(key, value)
            return {"success": True, "key": key, "length": length}
        except Exception as e:
            return {"error": str(e)}
    
    def lrange(self, key: str, start: int = 0, end: int = -1) -> Dict[str, Any]:
        """Get list range."""
        if not key:
            return {"error": "key is required"}
        try:
            values = self.client.lrange(key, start, end)
            return {"key": key, "values": values}
        except Exception as e:
            return {"error": str(e)}
    
    def info(self) -> Dict[str, Any]:
        """Get Redis info."""
        try:
            info = self.client.info()
            return {
                "redis_version": info.get("redis_version"),
                "connected_clients": info.get("connected_clients"),
                "used_memory_human": info.get("used_memory_human"),
                "total_keys": sum(info.get(f"db{i}", {}).get("keys", 0) for i in range(16)),
            }
        except Exception as e:
            return {"error": str(e)}


def redis_get(key: str) -> Dict[str, Any]:
    """Get Redis key."""
    return RedisTool().get(key=key)


def redis_set(key: str, value: str) -> Dict[str, Any]:
    """Set Redis key."""
    return RedisTool().set(key=key, value=value)



"""
Base classes for Recipe Tools.

Provides common infrastructure for all recipe tools including:
- Structured result types
- Dependency checking
- Logging
- Error handling
"""

import logging
import subprocess
import shutil
from abc import ABC, abstractmethod
from dataclasses import dataclass, field
from datetime import datetime
from pathlib import Path
from typing import Any, Dict, List, Optional, Union

logger = logging.getLogger(__name__)


class DependencyError(Exception):
    """Raised when a required dependency is missing."""
    
    def __init__(self, dependency: str, install_hint: str = ""):
        self.dependency = dependency
        self.install_hint = install_hint
        message = f"Missing dependency: {dependency}"
        if install_hint:
            message += f". Install with: {install_hint}"
        super().__init__(message)


@dataclass
class RecipeToolResult:
    """Base result class for recipe tool operations."""
    success: bool
    message: str = ""
    data: Any = None
    errors: List[str] = field(default_factory=list)
    warnings: List[str] = field(default_factory=list)
    duration_seconds: float = 0.0
    
    def to_dict(self) -> Dict[str, Any]:
        """Convert to dictionary for JSON serialization."""
        return {
            "success": self.success,
            "message": self.message,
            "data": self.data,
            "errors": self.errors,
            "warnings": self.warnings,
            "duration_seconds": self.duration_seconds,
        }


class RecipeToolBase(ABC):
    """
    Base class for all recipe tools.
    
    Subclasses must implement:
    - name: Tool name
    - description: Tool description
    - check_dependencies(): Verify required binaries/libraries
    """
    
    name: str = "base_tool"
    description: str = "Base recipe tool"
    
    def __init__(self, verbose: bool = False):
        self.verbose = verbose
        self._logger = logging.getLogger(f"{__name__}.{self.name}")
        if verbose:
            self._logger.setLevel(logging.DEBUG)
    
    @abstractmethod
    def check_dependencies(self) -> Dict[str, bool]:
        """
        Check if required dependencies are available.
        
        Returns:
            Dict mapping dependency name to availability status
        """
        pass
    
    def require_dependencies(self, deps: List[str]) -> None:
        """
        Raise DependencyError if any required dependencies are missing.
        
        Args:
            deps: List of dependency names to check
        """
        status = self.check_dependencies()
        missing = [d for d in deps if not status.get(d, False)]
        if missing:
            raise DependencyError(
                ", ".join(missing),
                self._get_install_hint(missing[0])
            )
    
    def _get_install_hint(self, dep: str) -> str:
        """Get installation hint for a dependency."""
        hints = {
            "ffmpeg": "brew install ffmpeg (macOS) or apt install ffmpeg (Linux)",
            "ffprobe": "brew install ffmpeg (macOS) or apt install ffmpeg (Linux)",
            "pdftotext": "brew install poppler (macOS) or apt install poppler-utils (Linux)",
            "pdfinfo": "brew install poppler (macOS) or apt install poppler-utils (Linux)",
            "pdfimages": "brew install poppler (macOS) or apt install poppler-utils (Linux)",
            "pandoc": "brew install pandoc (macOS) or apt install pandoc (Linux)",
            "convert": "brew install imagemagick (macOS) or apt install imagemagick (Linux)",
            "montage": "brew install imagemagick (macOS) or apt install imagemagick (Linux)",
            "git": "brew install git (macOS) or apt install git (Linux)",
            "tesseract": "brew install tesseract (macOS) or apt install tesseract-ocr (Linux)",
        }
        return hints.get(dep, f"pip install {dep}")
    
    def _check_binary(self, name: str) -> bool:
        """Check if a binary is available in PATH."""
        return shutil.which(name) is not None
    
    def _run_command(
        self,
        cmd: List[str],
        cwd: Optional[Path] = None,
        capture_output: bool = True,
        timeout: Optional[int] = None,
    ) -> subprocess.CompletedProcess:
        """
        Run a shell command safely.
        
        Args:
            cmd: Command and arguments
            cwd: Working directory
            capture_output: Whether to capture stdout/stderr
            timeout: Timeout in seconds
            
        Returns:
            CompletedProcess result
        """
        self._logger.debug(f"Running command: {' '.join(cmd)}")
        try:
            result = subprocess.run(
                cmd,
                cwd=cwd,
                capture_output=capture_output,
                text=True,
                timeout=timeout,
            )
            if result.returncode != 0 and self.verbose:
                self._logger.warning(f"Command failed: {result.stderr}")
            return result
        except subprocess.TimeoutExpired:
            raise TimeoutError(f"Command timed out after {timeout}s: {' '.join(cmd)}")
        except FileNotFoundError:
            raise DependencyError(cmd[0], self._get_install_hint(cmd[0]))
    
    def _ensure_output_dir(self, path: Union[str, Path]) -> Path:
        """Ensure output directory exists."""
        path = Path(path)
        if path.is_file():
            path = path.parent
        path.mkdir(parents=True, exist_ok=True)
        return path
    
    def _generate_output_path(
        self,
        input_path: Union[str, Path],
        output_dir: Optional[Union[str, Path]] = None,
        suffix: str = "",
        extension: Optional[str] = None,
    ) -> Path:
        """
        Generate output path based on input path.
        
        Args:
            input_path: Input file path
            output_dir: Output directory (default: same as input)
            suffix: Suffix to add before extension
            extension: New extension (default: keep original)
        """
        input_path = Path(input_path)
        if output_dir:
            output_dir = Path(output_dir)
            self._ensure_output_dir(output_dir)
        else:
            output_dir = input_path.parent
        
        stem = input_path.stem + suffix
        ext = extension if extension else input_path.suffix
        if not ext.startswith("."):
            ext = "." + ext
        
        return output_dir / (stem + ext)
    
    def _get_timestamp(self) -> str:
        """Get current timestamp for output naming."""
        return datetime.now().strftime("%Y%m%d-%H%M%S")
    
    def log(self, message: str, level: str = "info") -> None:
        """Log a message."""
        getattr(self._logger, level)(message)



"""Neo4j Tool for PraisonAI Agents.

Execute Cypher queries on Neo4j graph database.

Usage:
    from praisonai_tools import Neo4jTool
    
    neo4j = Neo4jTool()
    results = neo4j.query("MATCH (n) RETURN n LIMIT 10")

Environment Variables:
    NEO4J_URI: Neo4j connection URI
    NEO4J_USER: Username
    NEO4J_PASSWORD: Password
"""

import os
import logging
from typing import Any, Dict, List, Optional, Union

from praisonai_tools.tools.base import BaseTool

logger = logging.getLogger(__name__)


class Neo4jTool(BaseTool):
    """Tool for Neo4j graph database."""
    
    name = "neo4j"
    description = "Execute Cypher queries on Neo4j graph database."
    
    def __init__(
        self,
        uri: Optional[str] = None,
        user: Optional[str] = None,
        password: Optional[str] = None,
    ):
        self.uri = uri or os.getenv("NEO4J_URI", "bolt://localhost:7687")
        self.user = user or os.getenv("NEO4J_USER", "neo4j")
        self.password = password or os.getenv("NEO4J_PASSWORD")
        self._driver = None
        super().__init__()
    
    @property
    def driver(self):
        if self._driver is None:
            try:
                from neo4j import GraphDatabase
            except ImportError:
                raise ImportError("neo4j not installed. Install with: pip install neo4j")
            
            if not self.password:
                raise ValueError("NEO4J_PASSWORD required")
            
            self._driver = GraphDatabase.driver(self.uri, auth=(self.user, self.password))
        return self._driver
    
    def run(
        self,
        action: str = "query",
        cypher: Optional[str] = None,
        **kwargs
    ) -> Union[str, Dict[str, Any], List[Dict[str, Any]]]:
        action = action.lower().replace("-", "_")
        
        if action == "query":
            return self.query(cypher=cypher)
        elif action == "execute":
            return self.execute(cypher=cypher)
        else:
            return {"error": f"Unknown action: {action}"}
    
    def query(self, cypher: str) -> List[Dict[str, Any]]:
        """Execute Cypher query."""
        if not cypher:
            return [{"error": "cypher is required"}]
        
        try:
            with self.driver.session() as session:
                result = session.run(cypher)
                records = []
                for record in result:
                    records.append(dict(record))
                return records
        except Exception as e:
            logger.error(f"Neo4j query error: {e}")
            return [{"error": str(e)}]
    
    def execute(self, cypher: str) -> Dict[str, Any]:
        """Execute Cypher statement."""
        if not cypher:
            return {"error": "cypher is required"}
        
        try:
            with self.driver.session() as session:
                result = session.run(cypher)
                summary = result.consume()
                return {
                    "success": True,
                    "nodes_created": summary.counters.nodes_created,
                    "relationships_created": summary.counters.relationships_created,
                }
        except Exception as e:
            logger.error(f"Neo4j execute error: {e}")
            return {"error": str(e)}
    
    def close(self):
        if self._driver:
            self._driver.close()
            self._driver = None


def neo4j_query(cypher: str) -> List[Dict[str, Any]]:
    """Query Neo4j."""
    return Neo4jTool().query(cypher=cypher)



"""OpenBB Tool for PraisonAI Agents.

Financial data using OpenBB.

Usage:
    from praisonai_tools import OpenBBTool
    
    openbb = OpenBBTool()
    data = openbb.get_stock("AAPL")
"""

import logging
from typing import Any, Dict, List, Optional, Union

from praisonai_tools.tools.base import BaseTool

logger = logging.getLogger(__name__)


class OpenBBTool(BaseTool):
    """Tool for OpenBB financial data."""
    
    name = "openbb"
    description = "Get financial data using OpenBB."
    
    def run(
        self,
        action: str = "stock",
        symbol: Optional[str] = None,
        **kwargs
    ) -> Union[str, Dict[str, Any], List[Dict[str, Any]]]:
        action = action.lower().replace("-", "_")
        
        if action == "stock":
            return self.get_stock(symbol=symbol)
        elif action == "news":
            return self.get_news(symbol=symbol)
        elif action == "crypto":
            return self.get_crypto(symbol=symbol)
        else:
            return {"error": f"Unknown action: {action}"}
    
    def get_stock(self, symbol: str) -> Dict[str, Any]:
        """Get stock data."""
        if not symbol:
            return {"error": "symbol is required"}
        
        try:
            from openbb import obb
        except ImportError:
            return {"error": "openbb not installed. Install with: pip install openbb"}
        
        try:
            data = obb.equity.price.quote(symbol)
            return data.to_dict() if hasattr(data, "to_dict") else {"data": str(data)}
        except Exception as e:
            logger.error(f"OpenBB get_stock error: {e}")
            return {"error": str(e)}
    
    def get_news(self, symbol: str = None) -> List[Dict[str, Any]]:
        """Get financial news."""
        try:
            from openbb import obb
        except ImportError:
            return [{"error": "openbb not installed"}]
        
        try:
            if symbol:
                data = obb.news.company(symbol)
            else:
                data = obb.news.world()
            return data.to_list() if hasattr(data, "to_list") else [{"data": str(data)}]
        except Exception as e:
            logger.error(f"OpenBB get_news error: {e}")
            return [{"error": str(e)}]
    
    def get_crypto(self, symbol: str) -> Dict[str, Any]:
        """Get crypto data."""
        if not symbol:
            return {"error": "symbol is required"}
        
        try:
            from openbb import obb
        except ImportError:
            return {"error": "openbb not installed"}
        
        try:
            data = obb.crypto.price.historical(symbol)
            return data.to_dict() if hasattr(data, "to_dict") else {"data": str(data)}
        except Exception as e:
            logger.error(f"OpenBB get_crypto error: {e}")
            return {"error": str(e)}


def openbb_stock(symbol: str) -> Dict[str, Any]:
    """Get stock data."""
    return OpenBBTool().get_stock(symbol=symbol)



"""Serper Tool for PraisonAI Agents.

Google search using Serper API.

Usage:
    from praisonai_tools import SerperTool
    
    serper = SerperTool()
    results = serper.search("Python programming")

Environment Variables:
    SERPER_API_KEY: Serper API key
"""

import os
import logging
from typing import Any, Dict, List, Optional, Union

from praisonai_tools.tools.base import BaseTool

logger = logging.getLogger(__name__)


class SerperTool(BaseTool):
    """Tool for Google search using Serper."""
    
    name = "serper"
    description = "Search Google using Serper API."
    
    def __init__(self, api_key: Optional[str] = None):
        self.api_key = api_key or os.getenv("SERPER_API_KEY")
        super().__init__()
    
    def run(
        self,
        action: str = "search",
        query: Optional[str] = None,
        max_results: int = 10,
        **kwargs
    ) -> Union[str, Dict[str, Any], List[Dict[str, Any]]]:
        action = action.lower().replace("-", "_")
        
        if action == "search":
            return self.search(query=query, max_results=max_results)
        elif action == "news":
            return self.news(query=query, max_results=max_results)
        elif action == "images":
            return self.images(query=query, max_results=max_results)
        else:
            return {"error": f"Unknown action: {action}"}
    
    def _request(self, endpoint: str, query: str, num: int = 10) -> Dict:
        """Make Serper API request."""
        try:
            import requests
        except ImportError:
            return {"error": "requests not installed"}
        
        if not self.api_key:
            return {"error": "SERPER_API_KEY not configured"}
        
        try:
            response = requests.post(
                f"https://google.serper.dev/{endpoint}",
                headers={"X-API-KEY": self.api_key, "Content-Type": "application/json"},
                json={"q": query, "num": num},
                timeout=10,
            )
            return response.json()
        except Exception as e:
            return {"error": str(e)}
    
    def search(self, query: str, max_results: int = 10) -> List[Dict[str, Any]]:
        """Search Google."""
        if not query:
            return [{"error": "query is required"}]
        
        result = self._request("search", query, max_results)
        if "error" in result:
            return [result]
        
        results = []
        for item in result.get("organic", []):
            results.append({
                "title": item.get("title"),
                "url": item.get("link"),
                "snippet": item.get("snippet"),
                "position": item.get("position"),
            })
        return results
    
    def news(self, query: str, max_results: int = 10) -> List[Dict[str, Any]]:
        """Search Google News."""
        if not query:
            return [{"error": "query is required"}]
        
        result = self._request("news", query, max_results)
        if "error" in result:
            return [result]
        
        results = []
        for item in result.get("news", []):
            results.append({
                "title": item.get("title"),
                "url": item.get("link"),
                "source": item.get("source"),
                "date": item.get("date"),
                "snippet": item.get("snippet"),
            })
        return results
    
    def images(self, query: str, max_results: int = 10) -> List[Dict[str, Any]]:
        """Search Google Images."""
        if not query:
            return [{"error": "query is required"}]
        
        result = self._request("images", query, max_results)
        if "error" in result:
            return [result]
        
        results = []
        for item in result.get("images", []):
            results.append({
                "title": item.get("title"),
                "image_url": item.get("imageUrl"),
                "source": item.get("source"),
            })
        return results


def serper_search(query: str, max_results: int = 10) -> List[Dict[str, Any]]:
    """Search with Serper."""
    return SerperTool().search(query=query, max_results=max_results)
